# -*- coding: utf-8 -*-

# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import abc
import enum
from dataclasses import dataclass
from typing import Any, Dict, Optional


# This file mirrors the configuration options as defined in gs://google-cloud-aiplatform/schema/matchingengine/metadata/nearest_neighbor_search_1.0.0.yaml
class DistanceMeasureType(enum.Enum):
    """The distance measure used in nearest neighbor search."""

    # Dot Product Distance. Defined as a negative of the dot product
    DOT_PRODUCT_DISTANCE = "DOT_PRODUCT_DISTANCE"
    # Euclidean (L_2) Distance
    SQUARED_L2_DISTANCE = "SQUARED_L2_DISTANCE"
    # Manhattan (L_1) Distance
    L1_DISTANCE = "L1_DISTANCE"
    # Cosine Distance. Defined as 1 - cosine similarity.
    COSINE_DISTANCE = "COSINE_DISTANCE"


class FeatureNormType(enum.Enum):
    """Type of normalization to be carried out on each vector."""

    # Unit L2 normalization type.
    UNIT_L2_NORM = "UNIT_L2_NORM"
    # No normalization type is specified.
    NONE = "NONE"


class AlgorithmConfig(abc.ABC):
    """Base class for configuration options for matching algorithm."""

    def as_dict(self) -> Dict:
        """Returns the configuration as a dictionary.

        Returns:
            Dict[str, Any]
        """
        pass


@dataclass
class TreeAhConfig(AlgorithmConfig):
    """Configuration options for using the tree-AH algorithm (Shallow tree + Asymmetric Hashing).
    Please refer to this paper for more details: https://arxiv.org/abs/1908.10396

    Args:
        leaf_node_embedding_count (int):
            Optional. Number of embeddings on each leaf node. The default value is 1000 if not set.
        leaf_nodes_to_search_percent (float):
            The default percentage of leaf nodes that any query may be searched. Must be in
            range 1-100, inclusive. The default value is 10 (means 10%) if not set.
    """

    leaf_node_embedding_count: Optional[int] = None
    leaf_nodes_to_search_percent: Optional[float] = None

    def as_dict(self) -> Dict:
        """Returns the configuration as a dictionary.

        Returns:
            Dict[str, Any]
        """

        return {
            "treeAhConfig": {
                "leafNodeEmbeddingCount": self.leaf_node_embedding_count,
                "leafNodesToSearchPercent": self.leaf_nodes_to_search_percent,
            }
        }


@dataclass
class BruteForceConfig(AlgorithmConfig):
    """Configuration options for using brute force search, which simply
    implements the standard linear search in the database for each query.
    """

    def as_dict(self) -> Dict:
        """Returns the configuration as a dictionary.

        Returns:
            Dict[str, Any]
        """
        return {"bruteForceConfig": {}}


@dataclass
class MatchingEngineIndexConfig:
    """Configuration options for using the tree-AH algorithm (Shallow tree + Asymmetric Hashing).
    Please refer to this paper for more details: https://arxiv.org/abs/1908.10396

    Args:
        dimensions (int):
            Required. The number of dimensions of the input vectors.
        algorithm_config (AlgorithmConfig):
            Required. The configuration with regard to the algorithms used for efficient search.
        approximate_neighbors_count (int):
            Optional. The default number of neighbors to find via approximate search before exact reordering is
            performed. Exact reordering is a procedure where results returned by an
            approximate search algorithm are reordered via a more expensive distance computation.

            Required if tree-AH algorithm is used.
        distance_measure_type (DistanceMeasureType):
            Optional. The distance measure used in nearest neighbor search.
    """

    dimensions: int
    algorithm_config: AlgorithmConfig
    approximate_neighbors_count: Optional[int] = None
    distance_measure_type: Optional[DistanceMeasureType] = None

    def as_dict(self) -> Dict[str, Any]:
        """Returns the configuration as a dictionary.

        Returns:
            Dict[str, Any]
        """

        return {
            "dimensions": self.dimensions,
            "algorithmConfig": self.algorithm_config.as_dict(),
            "approximateNeighborsCount": self.approximate_neighbors_count,
            "distanceMeasureType": self.distance_measure_type,
        }



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from typing import MutableMapping, MutableSequence

import proto  # type: ignore

from google.cloud.aiplatform_v1beta1.types import pipeline_job as gca_pipeline_job
from google.cloud.aiplatform_v1beta1.types import (
    training_pipeline as gca_training_pipeline,
)
from google.protobuf import field_mask_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.cloud.aiplatform.v1beta1",
    manifest={
        "CreateTrainingPipelineRequest",
        "GetTrainingPipelineRequest",
        "ListTrainingPipelinesRequest",
        "ListTrainingPipelinesResponse",
        "DeleteTrainingPipelineRequest",
        "CancelTrainingPipelineRequest",
        "CreatePipelineJobRequest",
        "GetPipelineJobRequest",
        "ListPipelineJobsRequest",
        "ListPipelineJobsResponse",
        "DeletePipelineJobRequest",
        "CancelPipelineJobRequest",
    },
)


class CreateTrainingPipelineRequest(proto.Message):
    r"""Request message for
    [PipelineService.CreateTrainingPipeline][google.cloud.aiplatform.v1beta1.PipelineService.CreateTrainingPipeline].

    Attributes:
        parent (str):
            Required. The resource name of the Location to create the
            TrainingPipeline in. Format:
            ``projects/{project}/locations/{location}``
        training_pipeline (google.cloud.aiplatform_v1beta1.types.TrainingPipeline):
            Required. The TrainingPipeline to create.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    training_pipeline: gca_training_pipeline.TrainingPipeline = proto.Field(
        proto.MESSAGE,
        number=2,
        message=gca_training_pipeline.TrainingPipeline,
    )


class GetTrainingPipelineRequest(proto.Message):
    r"""Request message for
    [PipelineService.GetTrainingPipeline][google.cloud.aiplatform.v1beta1.PipelineService.GetTrainingPipeline].

    Attributes:
        name (str):
            Required. The name of the TrainingPipeline resource. Format:
            ``projects/{project}/locations/{location}/trainingPipelines/{training_pipeline}``
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ListTrainingPipelinesRequest(proto.Message):
    r"""Request message for
    [PipelineService.ListTrainingPipelines][google.cloud.aiplatform.v1beta1.PipelineService.ListTrainingPipelines].

    Attributes:
        parent (str):
            Required. The resource name of the Location to list the
            TrainingPipelines from. Format:
            ``projects/{project}/locations/{location}``
        filter (str):
            The standard list filter.

            Supported fields:

            -  ``display_name`` supports ``=``, ``!=`` comparisons, and
               ``:`` wildcard.
            -  ``state`` supports ``=``, ``!=`` comparisons.
            -  ``training_task_definition`` ``=``, ``!=`` comparisons,
               and ``:`` wildcard.
            -  ``create_time`` supports ``=``, ``!=``,\ ``<``,
               ``<=``,\ ``>``, ``>=`` comparisons. ``create_time`` must
               be in RFC 3339 format.
            -  ``labels`` supports general map functions that is:
               ``labels.key=value`` - key:value equality \`labels.key:\*
               - key existence

            Some examples of using the filter are:

            -  ``state="PIPELINE_STATE_SUCCEEDED" AND display_name:"my_pipeline_*"``
            -  ``state!="PIPELINE_STATE_FAILED" OR display_name="my_pipeline"``
            -  ``NOT display_name="my_pipeline"``
            -  ``create_time>"2021-05-18T00:00:00Z"``
            -  ``training_task_definition:"*automl_text_classification*"``
        page_size (int):
            The standard list page size.
        page_token (str):
            The standard list page token. Typically obtained via
            [ListTrainingPipelinesResponse.next_page_token][google.cloud.aiplatform.v1beta1.ListTrainingPipelinesResponse.next_page_token]
            of the previous
            [PipelineService.ListTrainingPipelines][google.cloud.aiplatform.v1beta1.PipelineService.ListTrainingPipelines]
            call.
        read_mask (google.protobuf.field_mask_pb2.FieldMask):
            Mask specifying which fields to read.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    filter: str = proto.Field(
        proto.STRING,
        number=2,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=3,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=4,
    )
    read_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=5,
        message=field_mask_pb2.FieldMask,
    )


class ListTrainingPipelinesResponse(proto.Message):
    r"""Response message for
    [PipelineService.ListTrainingPipelines][google.cloud.aiplatform.v1beta1.PipelineService.ListTrainingPipelines]

    Attributes:
        training_pipelines (MutableSequence[google.cloud.aiplatform_v1beta1.types.TrainingPipeline]):
            List of TrainingPipelines in the requested
            page.
        next_page_token (str):
            A token to retrieve the next page of results. Pass to
            [ListTrainingPipelinesRequest.page_token][google.cloud.aiplatform.v1beta1.ListTrainingPipelinesRequest.page_token]
            to obtain that page.
    """

    @property
    def raw_page(self):
        return self

    training_pipelines: MutableSequence[
        gca_training_pipeline.TrainingPipeline
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=gca_training_pipeline.TrainingPipeline,
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class DeleteTrainingPipelineRequest(proto.Message):
    r"""Request message for
    [PipelineService.DeleteTrainingPipeline][google.cloud.aiplatform.v1beta1.PipelineService.DeleteTrainingPipeline].

    Attributes:
        name (str):
            Required. The name of the TrainingPipeline resource to be
            deleted. Format:
            ``projects/{project}/locations/{location}/trainingPipelines/{training_pipeline}``
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class CancelTrainingPipelineRequest(proto.Message):
    r"""Request message for
    [PipelineService.CancelTrainingPipeline][google.cloud.aiplatform.v1beta1.PipelineService.CancelTrainingPipeline].

    Attributes:
        name (str):
            Required. The name of the TrainingPipeline to cancel.
            Format:
            ``projects/{project}/locations/{location}/trainingPipelines/{training_pipeline}``
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class CreatePipelineJobRequest(proto.Message):
    r"""Request message for
    [PipelineService.CreatePipelineJob][google.cloud.aiplatform.v1beta1.PipelineService.CreatePipelineJob].

    Attributes:
        parent (str):
            Required. The resource name of the Location to create the
            PipelineJob in. Format:
            ``projects/{project}/locations/{location}``
        pipeline_job (google.cloud.aiplatform_v1beta1.types.PipelineJob):
            Required. The PipelineJob to create.
        pipeline_job_id (str):
            The ID to use for the PipelineJob, which will become the
            final component of the PipelineJob name. If not provided, an
            ID will be automatically generated.

            This value should be less than 128 characters, and valid
            characters are /[a-z][0-9]-/.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    pipeline_job: gca_pipeline_job.PipelineJob = proto.Field(
        proto.MESSAGE,
        number=2,
        message=gca_pipeline_job.PipelineJob,
    )
    pipeline_job_id: str = proto.Field(
        proto.STRING,
        number=3,
    )


class GetPipelineJobRequest(proto.Message):
    r"""Request message for
    [PipelineService.GetPipelineJob][google.cloud.aiplatform.v1beta1.PipelineService.GetPipelineJob].

    Attributes:
        name (str):
            Required. The name of the PipelineJob resource. Format:
            ``projects/{project}/locations/{location}/pipelineJobs/{pipeline_job}``
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ListPipelineJobsRequest(proto.Message):
    r"""Request message for
    [PipelineService.ListPipelineJobs][google.cloud.aiplatform.v1beta1.PipelineService.ListPipelineJobs].

    Attributes:
        parent (str):
            Required. The resource name of the Location to list the
            PipelineJobs from. Format:
            ``projects/{project}/locations/{location}``
        filter (str):
            Lists the PipelineJobs that match the filter expression. The
            following fields are supported:

            -  ``pipeline_name``: Supports ``=`` and ``!=`` comparisons.
            -  ``display_name``: Supports ``=``, ``!=`` comparisons, and
               ``:`` wildcard.
            -  ``pipeline_job_user_id``: Supports ``=``, ``!=``
               comparisons, and ``:`` wildcard. for example, can check
               if pipeline's display_name contains *step* by doing
               display_name:"*step*"
            -  ``state``: Supports ``=`` and ``!=`` comparisons.
            -  ``create_time``: Supports ``=``, ``!=``, ``<``, ``>``,
               ``<=``, and ``>=`` comparisons. Values must be in RFC
               3339 format.
            -  ``update_time``: Supports ``=``, ``!=``, ``<``, ``>``,
               ``<=``, and ``>=`` comparisons. Values must be in RFC
               3339 format.
            -  ``end_time``: Supports ``=``, ``!=``, ``<``, ``>``,
               ``<=``, and ``>=`` comparisons. Values must be in RFC
               3339 format.
            -  ``labels``: Supports key-value equality and key presence.
            -  ``template_uri``: Supports ``=``, ``!=`` comparisons, and
               ``:`` wildcard.
            -  ``template_metadata.version``: Supports ``=``, ``!=``
               comparisons, and ``:`` wildcard.

            Filter expressions can be combined together using logical
            operators (``AND`` & ``OR``). For example:
            ``pipeline_name="test" AND create_time>"2020-05-18T13:30:00Z"``.

            The syntax to define filter expression is based on
            https://google.aip.dev/160.

            Examples:

            -  ``create_time>"2021-05-18T00:00:00Z" OR update_time>"2020-05-18T00:00:00Z"``
               PipelineJobs created or updated after 2020-05-18 00:00:00
               UTC.
            -  ``labels.env = "prod"`` PipelineJobs with label "env" set
               to "prod".
        page_size (int):
            The standard list page size.
        page_token (str):
            The standard list page token. Typically obtained via
            [ListPipelineJobsResponse.next_page_token][google.cloud.aiplatform.v1beta1.ListPipelineJobsResponse.next_page_token]
            of the previous
            [PipelineService.ListPipelineJobs][google.cloud.aiplatform.v1beta1.PipelineService.ListPipelineJobs]
            call.
        order_by (str):
            A comma-separated list of fields to order by. The default
            sort order is in ascending order. Use "desc" after a field
            name for descending. You can have multiple order_by fields
            provided e.g. "create_time desc, end_time", "end_time,
            start_time, update_time" For example, using "create_time
            desc, end_time" will order results by create time in
            descending order, and if there are multiple jobs having the
            same create time, order them by the end time in ascending
            order. if order_by is not specified, it will order by
            default order is create time in descending order. Supported
            fields:

            -  ``create_time``
            -  ``update_time``
            -  ``end_time``
            -  ``start_time``
        read_mask (google.protobuf.field_mask_pb2.FieldMask):
            Mask specifying which fields to read.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    filter: str = proto.Field(
        proto.STRING,
        number=2,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=3,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=4,
    )
    order_by: str = proto.Field(
        proto.STRING,
        number=6,
    )
    read_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=7,
        message=field_mask_pb2.FieldMask,
    )


class ListPipelineJobsResponse(proto.Message):
    r"""Response message for
    [PipelineService.ListPipelineJobs][google.cloud.aiplatform.v1beta1.PipelineService.ListPipelineJobs]

    Attributes:
        pipeline_jobs (MutableSequence[google.cloud.aiplatform_v1beta1.types.PipelineJob]):
            List of PipelineJobs in the requested page.
        next_page_token (str):
            A token to retrieve the next page of results. Pass to
            [ListPipelineJobsRequest.page_token][google.cloud.aiplatform.v1beta1.ListPipelineJobsRequest.page_token]
            to obtain that page.
    """

    @property
    def raw_page(self):
        return self

    pipeline_jobs: MutableSequence[gca_pipeline_job.PipelineJob] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=gca_pipeline_job.PipelineJob,
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class DeletePipelineJobRequest(proto.Message):
    r"""Request message for
    [PipelineService.DeletePipelineJob][google.cloud.aiplatform.v1beta1.PipelineService.DeletePipelineJob].

    Attributes:
        name (str):
            Required. The name of the PipelineJob resource to be
            deleted. Format:
            ``projects/{project}/locations/{location}/pipelineJobs/{pipeline_job}``
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class CancelPipelineJobRequest(proto.Message):
    r"""Request message for
    [PipelineService.CancelPipelineJob][google.cloud.aiplatform.v1beta1.PipelineService.CancelPipelineJob].

    Attributes:
        name (str):
            Required. The name of the PipelineJob to cancel. Format:
            ``projects/{project}/locations/{location}/pipelineJobs/{pipeline_job}``
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from google.cloud.aiplatform.helpers import container_uri_builders

get_prebuilt_prediction_container_uri = (
    container_uri_builders.get_prebuilt_prediction_container_uri
)
is_prebuilt_prediction_container_uri = (
    container_uri_builders.is_prebuilt_prediction_container_uri
)
_get_closest_match_prebuilt_container_uri = (
    container_uri_builders._get_closest_match_prebuilt_container_uri
)

__all__ = (
    "get_prebuilt_prediction_container_uri",
    "is_prebuilt_prediction_container_uri",
    "_get_closest_match_prebuilt_container_uri",
)



# -*- coding: utf-8 -*-

# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import pytest

from google.cloud.aiplatform import model_monitoring

from google.cloud.aiplatform_v1.types import (
    io as gca_io,
    model_monitoring as gca_model_monitoring,
)

_TEST_TARGET_FIELD = "target"
_TEST_BQ_DATASOURCE = "bq://test/data"
_TEST_GCS_DATASOURCE = "gs://test/data"
_TEST_OTHER_DATASOURCE = ""
_TEST_DRIFT_TRESHOLD = {"key": 0.2}
_TEST_EMAIL1 = "test1"
_TEST_EMAIL2 = "test2"
_TEST_VALID_DATA_FORMATS = ["tf-record", "csv", "jsonl"]
_TEST_SAMPLING_RATE = 0.8
_TEST_MONITORING_INTERVAL = 1
_TEST_SKEW_THRESHOLDS = [None, 0.2, {"key": 0.1}]
_TEST_ATTRIBUTE_SKEW_THRESHOLDS = [None, {"key": 0.1}]


class TestModelMonitoringConfigs:
    """Tests for model monitoring configs."""

    @pytest.mark.parametrize(
        "data_source",
        [_TEST_BQ_DATASOURCE, _TEST_GCS_DATASOURCE, _TEST_OTHER_DATASOURCE],
    )
    @pytest.mark.parametrize("data_format", _TEST_VALID_DATA_FORMATS)
    @pytest.mark.parametrize("skew_thresholds", _TEST_SKEW_THRESHOLDS)
    def test_skew_config_proto_value(self, data_source, data_format, skew_thresholds):
        """Tests if skew config can be constrctued properly to gapic proto."""
        attribute_skew_thresholds = {"key": 0.1}
        skew_config = model_monitoring.SkewDetectionConfig(
            data_source=data_source,
            skew_thresholds=skew_thresholds,
            target_field=_TEST_TARGET_FIELD,
            attribute_skew_thresholds=attribute_skew_thresholds,
            data_format=data_format,
        )
        # data_format and data source are not used at
        # TrainingPredictionSkewDetectionConfig.
        if isinstance(skew_thresholds, dict):
            expected_gapic_proto = gca_model_monitoring.ModelMonitoringObjectiveConfig.TrainingPredictionSkewDetectionConfig(
                skew_thresholds={
                    key: gca_model_monitoring.ThresholdConfig(value=val)
                    for key, val in skew_thresholds.items()
                },
                attribution_score_skew_thresholds={
                    key: gca_model_monitoring.ThresholdConfig(value=val)
                    for key, val in attribute_skew_thresholds.items()
                },
            )
        else:
            expected_gapic_proto = gca_model_monitoring.ModelMonitoringObjectiveConfig.TrainingPredictionSkewDetectionConfig(
                default_skew_threshold=gca_model_monitoring.ThresholdConfig(
                    value=skew_thresholds
                )
                if skew_thresholds is not None
                else None,
                attribution_score_skew_thresholds={
                    key: gca_model_monitoring.ThresholdConfig(value=val)
                    for key, val in attribute_skew_thresholds.items()
                },
            )
        assert skew_config.as_proto() == expected_gapic_proto

    @pytest.mark.parametrize(
        "data_source",
        [_TEST_BQ_DATASOURCE, _TEST_GCS_DATASOURCE, _TEST_OTHER_DATASOURCE],
    )
    @pytest.mark.parametrize("data_format", _TEST_VALID_DATA_FORMATS)
    @pytest.mark.parametrize("skew_thresholds", _TEST_SKEW_THRESHOLDS)
    @pytest.mark.parametrize(
        "attribute_skew_thresholds", _TEST_ATTRIBUTE_SKEW_THRESHOLDS
    )
    def test_valid_configs(
        self, data_source, data_format, skew_thresholds, attribute_skew_thresholds
    ):
        """Test config creation validity."""
        random_sample_config = model_monitoring.RandomSampleConfig(
            sample_rate=_TEST_SAMPLING_RATE
        )

        schedule_config = model_monitoring.ScheduleConfig(
            monitor_interval=_TEST_MONITORING_INTERVAL
        )

        alert_config = model_monitoring.EmailAlertConfig(
            user_emails=[_TEST_EMAIL1, _TEST_EMAIL2]
        )

        prediction_drift_config = model_monitoring.DriftDetectionConfig(
            drift_thresholds=_TEST_DRIFT_TRESHOLD
        )

        skew_config = model_monitoring.SkewDetectionConfig(
            data_source=data_source,
            skew_thresholds=skew_thresholds,
            target_field=_TEST_TARGET_FIELD,
            attribute_skew_thresholds=attribute_skew_thresholds,
            data_format=data_format,
        )
        expected_training_dataset = (
            gca_model_monitoring.ModelMonitoringObjectiveConfig.TrainingDataset(
                bigquery_source=gca_io.BigQuerySource(input_uri=_TEST_BQ_DATASOURCE),
                target_field=_TEST_TARGET_FIELD,
            )
        )

        xai_config = model_monitoring.ExplanationConfig()

        objective_config = model_monitoring.ObjectiveConfig(
            skew_detection_config=skew_config,
            drift_detection_config=prediction_drift_config,
            explanation_config=xai_config,
        )

        if data_source == _TEST_BQ_DATASOURCE:
            assert (
                objective_config.as_proto().training_dataset
                == expected_training_dataset
            )
        assert (
            objective_config.as_proto().training_prediction_skew_detection_config
            == skew_config.as_proto()
        )
        assert (
            objective_config.as_proto().prediction_drift_detection_config
            == prediction_drift_config.as_proto()
        )
        assert objective_config.as_proto().explanation_config == xai_config.as_proto()
        assert _TEST_EMAIL1 in alert_config.as_proto().email_alert_config.user_emails
        assert _TEST_EMAIL2 in alert_config.as_proto().email_alert_config.user_emails
        assert (
            random_sample_config.as_proto().random_sample_config.sample_rate
            == _TEST_SAMPLING_RATE
        )
        assert (
            schedule_config.as_proto().monitor_interval.seconds
            == _TEST_MONITORING_INTERVAL * 3600
        )

    @pytest.mark.parametrize("data_source", [_TEST_GCS_DATASOURCE])
    @pytest.mark.parametrize("data_format", ["other"])
    @pytest.mark.parametrize("skew_thresholds", _TEST_SKEW_THRESHOLDS)
    @pytest.mark.parametrize(
        "attribute_skew_thresholds", _TEST_ATTRIBUTE_SKEW_THRESHOLDS
    )
    def test_invalid_data_format(
        self, data_source, data_format, skew_thresholds, attribute_skew_thresholds
    ):
        if data_format == "other":
            with pytest.raises(ValueError) as e:
                model_monitoring.ObjectiveConfig(
                    skew_detection_config=model_monitoring.SkewDetectionConfig(
                        data_source=data_source,
                        skew_thresholds=skew_thresholds,
                        target_field=_TEST_TARGET_FIELD,
                        attribute_skew_thresholds=attribute_skew_thresholds,
                        data_format=data_format,
                    )
                ).as_proto()
            assert (
                "Unsupported value in skew detection config. `data_format` must be one of tf-record, csv, or jsonl"
                in str(e.value)
            )



# -*- coding: utf-8 -*-

# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from abc import ABC, abstractmethod
from typing import Any


class Predictor(ABC):
    """Interface of the Predictor class for Custom Prediction Routines.

    The Predictor is responsible for the ML logic for processing a prediction request.
    Specifically, the Predictor must define:
    (1) How to load all model artifacts used during prediction into memory.
    (2) The logic that should be executed at predict time.

    When using the default ``PredictionHandler``, the ``Predictor`` will be invoked as
    follows:

    .. code-block:: python

        predictor.postprocess(predictor.predict(predictor.preprocess(prediction_input)))

    """

    def __init__(self):
        return

    @abstractmethod
    def load(self, artifacts_uri: str) -> None:
        """Loads the model artifact.

        Args:
            artifacts_uri (str):
                Required. The value of the environment variable AIP_STORAGE_URI.
        """
        pass

    def preprocess(self, prediction_input: Any) -> Any:
        """Preprocesses the prediction input before doing the prediction.

        Args:
            prediction_input (Any):
                Required. The prediction input that needs to be preprocessed.

        Returns:
            The preprocessed prediction input.
        """
        return prediction_input

    @abstractmethod
    def predict(self, instances: Any) -> Any:
        """Performs prediction.

        Args:
            instances (Any):
                Required. The instance(s) used for performing prediction.

        Returns:
            Prediction results.
        """
        pass

    def postprocess(self, prediction_results: Any) -> Any:
        """Postprocesses the prediction results.

        Args:
            prediction_results (Any):
                Required. The prediction results.

        Returns:
            The postprocessed prediction results.
        """
        return prediction_results



# -*- coding: utf-8 -*-

# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import pytest

from google.cloud import aiplatform
from tests.system.aiplatform import e2e_base

from google.protobuf.json_format import MessageToDict


@pytest.mark.usefixtures("tear_down_resources")
class TestPipelineJob(e2e_base.TestEndToEnd):

    _temp_prefix = "tmpvrtxsdk-e2e"

    def test_add_pipeline_job_to_experiment(self, shared_state):
        from kfp import components

        # Components:
        def train(
            number_of_epochs: int,
            learning_rate: float,
        ):
            print(f"number_of_epochs={number_of_epochs}")
            print(f"learning_rate={learning_rate}")

        train_op = components.create_component_from_func(train)

        # Pipeline:
        def training_pipeline(number_of_epochs: int = 10):
            train_op(
                number_of_epochs=number_of_epochs,
                learning_rate="0.1",
            )

        # Submitting the pipeline:
        aiplatform.init(
            project=e2e_base._PROJECT,
            location=e2e_base._LOCATION,
        )
        job = aiplatform.PipelineJob.from_pipeline_func(
            pipeline_func=training_pipeline,
        )
        job.submit()

        shared_state.setdefault("resources", []).append(job)

        job.wait()

        list_with_read_mask = aiplatform.PipelineJob.list(enable_simple_view=True)
        list_without_read_mask = aiplatform.PipelineJob.list()

        # enable_simple_view=True should apply the `read_mask` filter to limit PipelineJob fields returned
        assert "serviceAccount" in MessageToDict(
            list_without_read_mask[0].gca_resource._pb
        )
        assert "serviceAccount" not in MessageToDict(
            list_with_read_mask[0].gca_resource._pb
        )



# -*- coding: utf-8 -*-

# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import copy

from typing import Optional, TypeVar, Mapping, Any
from google.cloud.aiplatform.vizier.client_abc import TrialInterface

from google.auth import credentials as auth_credentials
from google.cloud.aiplatform import base
from google.cloud.aiplatform import utils
from google.cloud.aiplatform.vizier import study
from google.cloud.aiplatform.vizier import pyvizier as vz

_T = TypeVar("_T")
_LOGGER = base.Logger(__name__)


class Trial(base.VertexAiResourceNounWithFutureManager, TrialInterface):
    """Manage Trial resource for Vertex Vizier."""

    client_class = utils.VizierClientWithOverride

    _resource_noun = "trial"
    _getter_method = "get_trial"
    _list_method = "list_trials"
    _delete_method = "delete_trial"
    _parse_resource_name_method = "parse_trial_path"
    _format_resource_name_method = "trial_path"

    def __init__(
        self,
        trial_name: str,
        study_id: Optional[str] = None,
        project: Optional[str] = None,
        location: Optional[str] = None,
        credentials: Optional[auth_credentials.Credentials] = None,
    ):
        """Retrieves an existing managed trial given a trial resource name or a study id.

        Example Usage:
            trial = aiplatform.Trial(trial_name = 'projects/123/locations/us-central1/studies/12345678/trials/1')
            or
            trial = aiplatform.Trial(trial_name = '1', study_id = '12345678')

        Args:
            trial_name (str):
                Required. A fully-qualified trial resource name or a trial ID.
                Example: "projects/123/locations/us-central1/studies/12345678/trials/1" or "12345678" when
                project and location are initialized or passed.
            study_id (str):
                Optional. A fully-qualified study resource name or a study ID.
                Example: "projects/123/locations/us-central1/studies/12345678" or "12345678" when
                project and location are initialized or passed.
            project (str):
                Optional. Project to retrieve trial from. If not set, project
                set in aiplatform.init will be used.
            location (str):
                Optional. Location to retrieve trial from. If not set, location
                set in aiplatform.init will be used.
            credentials (auth_credentials.Credentials):
                Optional. Custom credentials to use to retrieve this Feature. Overrides
                credentials set in aiplatform.init.
        """

        base.VertexAiResourceNounWithFutureManager.__init__(
            self,
            project=project,
            location=location,
            credentials=credentials,
            resource_name=trial_name,
        )
        self._gca_resource = self._get_gca_resource(
            resource_name=trial_name,
            parent_resource_name_fields={
                study.Study._resource_noun: study_id,
            }
            if study_id
            else study_id,
        )

    @property
    def uid(self) -> int:
        """Unique identifier of the trial."""
        trial_path_components = self._parse_resource_name(self.resource_name)
        return int(trial_path_components["trial"])

    @property
    def parameters(self) -> Mapping[str, Any]:
        """Parameters of the trial."""
        trial = self.api_client.get_trial(name=self.resource_name)
        return vz.TrialConverter.from_proto(trial).parameters

    @property
    def status(self) -> vz.TrialStatus:
        """Status of the Trial."""
        trial = self.api_client.get_trial(name=self.resource_name)
        return vz.TrialConverter.from_proto(trial).status

    def delete(self) -> None:
        """Deletes the Trial in Vizier service."""
        self.api_client.delete_trial(name=self.resource_name)

    def complete(
        self,
        measurement: Optional[vz.Measurement] = None,
        *,
        infeasible_reason: Optional[str] = None
    ) -> Optional[vz.Measurement]:
        """Completes the trial and #materializes the measurement.

        * If `measurement` is provided, then Vizier writes it as the trial's final
        measurement and returns it.
        * If `infeasible_reason` is provided, `measurement` is not needed.
        * If neither is provided, then Vizier selects an existing (intermediate)
        measurement to be the final measurement and returns it.

        Args:
          measurement: Final measurement.
          infeasible_reason: Indefeasibly reason for missing final measurement.
        """
        complete_trial_request = {"name": self.resource_name}
        if infeasible_reason is not None:
            complete_trial_request["infeasible_reason"] = infeasible_reason
            complete_trial_request["trial_infeasible"] = True
        if measurement is not None:
            complete_trial_request[
                "final_measurement"
            ] = vz.MeasurementConverter.to_proto(measurement)
        trial = self.api_client.complete_trial(request=complete_trial_request)
        return (
            vz.MeasurementConverter.from_proto(trial.final_measurement)
            if trial.final_measurement
            else None
        )

    def should_stop(self) -> bool:
        """Returns true if the Trial should stop."""
        check_trial_early_stopping_state_request = {"trial_name": self.resource_name}
        should_stop_lro = self.api_client.check_trial_early_stopping_state(
            request=check_trial_early_stopping_state_request
        )
        _LOGGER.log_action_started_against_resource_with_lro(
            "ShouldStop", "trial", self.__class__, should_stop_lro
        )
        should_stop_lro.result()
        _LOGGER.log_action_completed_against_resource("trial", "should_stop", self)
        return should_stop_lro.result().should_stop

    def add_measurement(self, measurement: vz.Measurement) -> None:
        """Adds an intermediate measurement."""
        add_trial_measurement_request = {
            "trial_name": self.resource_name,
        }
        add_trial_measurement_request["measurement"] = vz.MeasurementConverter.to_proto(
            measurement
        )
        self.api_client.add_trial_measurement(request=add_trial_measurement_request)

    def materialize(self, *, include_all_measurements: bool = True) -> vz.Trial:
        """#Materializes the Trial.

        Args:
          include_all_measurements: If True, returned Trial includes all
            intermediate measurements. The final measurement is always provided.
        """
        trial = self.api_client.get_trial(name=self.resource_name)
        return copy.deepcopy(vz.TrialConverter.from_proto(trial))



# -*- coding: utf-8 -*-

# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import json
import pytest

from importlib import reload
from unittest import mock
from unittest.mock import patch

from google.cloud import aiplatform
from google.cloud.aiplatform import initializer
from google.cloud.aiplatform.compat.types import artifact as gca_artifact
from google.cloud.aiplatform.compat.types import execution as gca_execution
from google.cloud.aiplatform.metadata import metadata
from google.cloud.aiplatform.metadata.schema import base_artifact
from google.cloud.aiplatform.metadata.schema import base_execution
from google.cloud.aiplatform.metadata.schema import base_context
from google.cloud.aiplatform.metadata.schema.google import (
    artifact_schema as google_artifact_schema,
)
from google.cloud.aiplatform.metadata.schema.system import (
    artifact_schema as system_artifact_schema,
)
from google.cloud.aiplatform.metadata.schema.system import (
    context_schema as system_context_schema,
)
from google.cloud.aiplatform.metadata.schema.system import (
    execution_schema as system_execution_schema,
)
from google.cloud.aiplatform.metadata.schema import utils
from google.cloud.aiplatform_v1 import MetadataServiceClient
from google.cloud.aiplatform_v1 import Artifact as GapicArtifact
from google.cloud.aiplatform_v1 import Execution as GapicExecution


# project
_TEST_PROJECT = "test-project"
_TEST_LOCATION = "us-central1"
_TEST_METADATA_STORE = "test-metadata-store"
_TEST_PARENT = f"projects/{_TEST_PROJECT}/locations/{_TEST_LOCATION}"

# resource attributes
_TEST_ARTIFACT_STATE = gca_artifact.Artifact.State.STATE_UNSPECIFIED
_TEST_EXECUTION_STATE = gca_execution.Execution.State.STATE_UNSPECIFIED
_TEST_URI = "test-uri"
_TEST_DISPLAY_NAME = "test-display-name"
_TEST_SCHEMA_TITLE = "test.Example"
_TEST_SCHEMA_VERSION = "0.0.1"
_TEST_DESCRIPTION = "test description"
_TEST_METADATA = {"test-param1": 1, "test-param2": "test-value", "test-param3": True}
_TEST_UPDATED_METADATA = {
    "test-param1": 2.0,
    "test-param2": "test-value-1",
    "test-param3": False,
}

# artifact
_TEST_ARTIFACT_ID = "test-artifact-id"
_TEST_ARTIFACT_NAME = f"{_TEST_PARENT}/metadataStores/{_TEST_METADATA_STORE}/artifacts/{_TEST_ARTIFACT_ID}"

# execution
_TEST_EXECUTION_ID = "test-execution-id"
_TEST_EXECUTION_NAME = f"{_TEST_PARENT}/metadataStores/{_TEST_METADATA_STORE}/executions/{_TEST_EXECUTION_ID}"

# context
_TEST_CONTEXT_ID = "test-context-id"
_TEST_CONTEXT_NAME = (
    f"{_TEST_PARENT}/metadataStores/{_TEST_METADATA_STORE}/contexts/{_TEST_CONTEXT_ID}"
)


@pytest.fixture
def get_artifact_mock():
    with patch.object(MetadataServiceClient, "get_artifact") as get_artifact_mock:
        get_artifact_mock.return_value = GapicArtifact(
            name=_TEST_ARTIFACT_NAME,
            display_name=_TEST_DISPLAY_NAME,
            schema_title=_TEST_SCHEMA_TITLE,
            schema_version=_TEST_SCHEMA_VERSION,
            description=_TEST_DESCRIPTION,
            metadata=_TEST_METADATA,
            state=GapicArtifact.State.STATE_UNSPECIFIED,
        )
        yield get_artifact_mock


@pytest.fixture
def initializer_create_client_mock():
    with patch.object(
        initializer.global_config, "create_client"
    ) as initializer_create_client_mock:
        yield initializer_create_client_mock


@pytest.fixture
def base_artifact_init_with_resouce_name_mock():
    with patch.object(
        base_artifact.BaseArtifactSchema, "_init_with_resource_name"
    ) as base_artifact_init_with_resouce_name_mock:
        yield base_artifact_init_with_resouce_name_mock


@pytest.fixture
def base_execution_init_with_resouce_name_mock():
    with patch.object(
        base_execution.BaseExecutionSchema, "_init_with_resource_name"
    ) as base_execution_init_with_resouce_name_mock:
        yield base_execution_init_with_resouce_name_mock


@pytest.fixture
def base_context_init_with_resouce_name_mock():
    with patch.object(
        base_context.BaseContextSchema, "_init_with_resource_name"
    ) as base_context_init_with_resouce_name_mock:
        yield base_context_init_with_resouce_name_mock


@pytest.fixture
def get_execution_mock():
    with patch.object(MetadataServiceClient, "get_execution") as get_execution_mock:
        get_execution_mock.return_value = GapicExecution(
            name=_TEST_EXECUTION_NAME,
            display_name=_TEST_DISPLAY_NAME,
            schema_title=_TEST_SCHEMA_TITLE,
            schema_version=_TEST_SCHEMA_VERSION,
            description=_TEST_DESCRIPTION,
            metadata=_TEST_METADATA,
            state=GapicExecution.State.RUNNING,
        )
        yield get_execution_mock


@pytest.fixture
def get_context_mock():
    with patch.object(MetadataServiceClient, "get_context") as get_context_mock:
        get_context_mock.return_value = GapicExecution(
            name=_TEST_CONTEXT_NAME,
            display_name=_TEST_DISPLAY_NAME,
            schema_title=_TEST_SCHEMA_TITLE,
            schema_version=_TEST_SCHEMA_VERSION,
            description=_TEST_DESCRIPTION,
            metadata=_TEST_METADATA,
        )
        yield get_context_mock


@pytest.fixture
def create_artifact_mock():
    with patch.object(MetadataServiceClient, "create_artifact") as create_artifact_mock:
        create_artifact_mock.return_value = GapicArtifact(
            name=_TEST_ARTIFACT_NAME,
            display_name=_TEST_DISPLAY_NAME,
            schema_title=_TEST_SCHEMA_TITLE,
            schema_version=_TEST_SCHEMA_VERSION,
            description=_TEST_DESCRIPTION,
            metadata=_TEST_METADATA,
            state=GapicArtifact.State.STATE_UNSPECIFIED,
        )
        yield create_artifact_mock


@pytest.fixture
def create_execution_mock():
    with patch.object(
        MetadataServiceClient, "create_execution"
    ) as create_execution_mock:
        create_execution_mock.return_value = GapicExecution(
            name=_TEST_EXECUTION_NAME,
            display_name=_TEST_DISPLAY_NAME,
            schema_title=_TEST_SCHEMA_TITLE,
            schema_version=_TEST_SCHEMA_VERSION,
            description=_TEST_DESCRIPTION,
            metadata=_TEST_METADATA,
            state=GapicExecution.State.RUNNING,
        )
        yield create_execution_mock


@pytest.fixture
def create_context_mock():
    with patch.object(MetadataServiceClient, "create_context") as create_context_mock:
        create_context_mock.return_value = GapicExecution(
            name=_TEST_CONTEXT_NAME,
            display_name=_TEST_DISPLAY_NAME,
            schema_title=_TEST_SCHEMA_TITLE,
            schema_version=_TEST_SCHEMA_VERSION,
            description=_TEST_DESCRIPTION,
            metadata=_TEST_METADATA,
        )
        yield create_context_mock


@pytest.fixture
def list_artifacts_mock():
    with patch.object(MetadataServiceClient, "list_artifacts") as list_artifacts_mock:
        list_artifacts_mock.return_value = []
        yield list_artifacts_mock


@pytest.fixture
def list_executions_mock():
    with patch.object(MetadataServiceClient, "list_executions") as list_executions_mock:
        list_executions_mock.return_value = []
        yield list_executions_mock


@pytest.fixture
def list_contexts_mock():
    with patch.object(MetadataServiceClient, "list_contexts") as list_contexts_mock:
        list_contexts_mock.return_value = []
        yield list_contexts_mock


@pytest.mark.usefixtures("google_auth_mock")
class TestMetadataBaseArtifactSchema:
    def setup_method(self):
        reload(initializer)
        reload(metadata)
        reload(aiplatform)

    def teardown_method(self):
        initializer.global_pool.shutdown(wait=True)

    def test_base_class_instatiated_uses_schema_title(self):
        class TestArtifact(base_artifact.BaseArtifactSchema):
            schema_title = _TEST_SCHEMA_TITLE

        artifact = TestArtifact()
        assert artifact.schema_title == _TEST_SCHEMA_TITLE

    def test_base_class_print_output(self, capsys):
        class TestArtifact(base_artifact.BaseArtifactSchema):
            schema_title = _TEST_SCHEMA_TITLE

        artifact = TestArtifact()
        print(artifact)
        captured = capsys.readouterr()
        assert (
            captured.out
            == f"{object.__repr__(artifact)}\n"
            + f"schema_title: {_TEST_SCHEMA_TITLE}\n"
        )

    def test_base_class_inherited_methods_error(self):
        class TestArtifact(base_artifact.BaseArtifactSchema):
            schema_title = _TEST_SCHEMA_TITLE

        artifact = TestArtifact()

        with pytest.raises(RuntimeError) as exception:
            artifact.resource_name
        assert str(exception.value) == "TestArtifact resource has not been created."

        with pytest.raises(RuntimeError) as exception:
            artifact.lineage_console_uri
        assert str(exception.value) == "TestArtifact resource has not been created."

        with pytest.raises(RuntimeError) as exception:
            artifact.sync_resource()
        assert str(exception.value) == "TestArtifact resource has not been created."

        with pytest.raises(RuntimeError) as exception:
            artifact.update(
                metadata=_TEST_UPDATED_METADATA,
                description=_TEST_DESCRIPTION,
            )
        assert str(exception.value) == "TestArtifact resource has not been created."

    def test_base_class_parameters_overrides_default_values(self):
        class TestArtifact(base_artifact.BaseArtifactSchema):
            schema_title = _TEST_SCHEMA_TITLE

        artifact = TestArtifact(
            state=_TEST_ARTIFACT_STATE,
            schema_version=_TEST_SCHEMA_VERSION,
            artifact_id=_TEST_ARTIFACT_ID,
            uri=_TEST_URI,
            display_name=_TEST_DISPLAY_NAME,
            description=_TEST_DESCRIPTION,
            metadata=_TEST_UPDATED_METADATA,
        )
        assert artifact.state == _TEST_ARTIFACT_STATE
        assert artifact.state == _TEST_ARTIFACT_STATE
        assert artifact.schema_version == _TEST_SCHEMA_VERSION
        assert artifact.artifact_id == _TEST_ARTIFACT_ID
        assert artifact.schema_title == _TEST_SCHEMA_TITLE
        assert artifact.uri == _TEST_URI
        assert artifact.display_name == _TEST_DISPLAY_NAME
        assert artifact.description == _TEST_DESCRIPTION
        assert artifact.metadata == _TEST_UPDATED_METADATA

    def test_base_class_without_schema_title_raises_error(self):
        with pytest.raises(TypeError):
            base_artifact.BaseArtifactSchema()

    @pytest.mark.usefixtures("create_artifact_mock", "get_artifact_mock")
    def test_create_is_called_with_default_parameters(self, create_artifact_mock):
        aiplatform.init(project=_TEST_PROJECT)

        class TestArtifact(base_artifact.BaseArtifactSchema):
            schema_title = _TEST_SCHEMA_TITLE

        artifact = TestArtifact(
            uri=_TEST_URI,
            display_name=_TEST_DISPLAY_NAME,
            description=_TEST_DESCRIPTION,
            metadata=_TEST_UPDATED_METADATA,
            state=_TEST_ARTIFACT_STATE,
        )
        artifact.create(metadata_store_id=_TEST_METADATA_STORE)
        create_artifact_mock.assert_called_once_with(
            parent=f"{_TEST_PARENT}/metadataStores/{_TEST_METADATA_STORE}",
            artifact=mock.ANY,
            artifact_id=None,
        )
        _, _, kwargs = create_artifact_mock.mock_calls[0]
        assert kwargs["artifact"].schema_title == _TEST_SCHEMA_TITLE
        assert kwargs["artifact"].uri == _TEST_URI
        assert kwargs["artifact"].display_name == _TEST_DISPLAY_NAME
        assert kwargs["artifact"].description == _TEST_DESCRIPTION
        assert kwargs["artifact"].metadata == _TEST_UPDATED_METADATA
        assert kwargs["artifact"].state == _TEST_ARTIFACT_STATE

    @pytest.mark.usefixtures(
        "base_artifact_init_with_resouce_name_mock",
        "initializer_create_client_mock",
        "create_artifact_mock",
        "get_artifact_mock",
    )
    def test_artifact_create_call_sets_the_user_agent_header(
        self, initializer_create_client_mock
    ):
        aiplatform.init(project=_TEST_PROJECT)

        class TestArtifact(base_artifact.BaseArtifactSchema):
            schema_title = _TEST_SCHEMA_TITLE

        artifact = TestArtifact(
            uri=_TEST_URI,
            display_name=_TEST_DISPLAY_NAME,
            description=_TEST_DESCRIPTION,
            metadata=_TEST_UPDATED_METADATA,
            state=_TEST_ARTIFACT_STATE,
        )
        artifact.create()
        _, _, kwargs = initializer_create_client_mock.mock_calls[0]
        assert kwargs["appended_user_agent"] == [
            "sdk_command/aiplatform.metadata.schema.base_artifact.BaseArtifactSchema.create"
        ]

    @pytest.mark.usefixtures(
        "initializer_create_client_mock",
        "create_artifact_mock",
        "get_artifact_mock",
    )
    def test_artifact_init_call_sets_the_user_agent_header(
        self, initializer_create_client_mock
    ):
        aiplatform.init(project=_TEST_PROJECT)

        class TestArtifact(base_artifact.BaseArtifactSchema):
            schema_title = _TEST_SCHEMA_TITLE

        artifact = TestArtifact(
            uri=_TEST_URI,
            display_name=_TEST_DISPLAY_NAME,
            description=_TEST_DESCRIPTION,
            metadata=_TEST_UPDATED_METADATA,
            state=_TEST_ARTIFACT_STATE,
        )
        artifact._init_with_resource_name(artifact_name=_TEST_ARTIFACT_NAME)
        _, _, kwargs = initializer_create_client_mock.mock_calls[0]
        assert kwargs["appended_user_agent"] == [
            "sdk_command/aiplatform.metadata.schema.base_artifact.BaseArtifactSchema._init_with_resource_name"
        ]

    def test_list_artifacts(self, list_artifacts_mock):
        aiplatform.init(project=_TEST_PROJECT, location=_TEST_LOCATION)

        class TestArtifact(base_artifact.BaseArtifactSchema):
            schema_title = _TEST_SCHEMA_TITLE

        TestArtifact.list()
        list_artifacts_mock.assert_called_once_with(
            request={
                "parent": f"{_TEST_PARENT}/metadataStores/default",
                "filter": f'schema_title="{_TEST_SCHEMA_TITLE}"',
            }
        )


@pytest.mark.usefixtures("google_auth_mock")
class TestMetadataBaseExecutionSchema:
    def setup_method(self):
        reload(initializer)
        reload(metadata)
        reload(aiplatform)

    def teardown_method(self):
        initializer.global_pool.shutdown(wait=True)

    def test_base_class_overrides_default_schema_title(self):
        class TestExecution(base_execution.BaseExecutionSchema):
            schema_title = _TEST_SCHEMA_TITLE

        execution = TestExecution()
        assert execution.schema_title == _TEST_SCHEMA_TITLE

    def test_base_class_print_output(self, capsys):
        class TestExecution(base_execution.BaseExecutionSchema):
            schema_title = _TEST_SCHEMA_TITLE

        execution = TestExecution()
        print(execution)
        captured = capsys.readouterr()
        assert (
            captured.out
            == f"{object.__repr__(execution)}\n"
            + f"schema_title: {_TEST_SCHEMA_TITLE}\n"
        )

    def test_base_class_inherited_methods_error(self):
        class TestExecution(base_execution.BaseExecutionSchema):
            schema_title = _TEST_SCHEMA_TITLE

        execution = TestExecution()

        with pytest.raises(RuntimeError) as exception:
            execution.resource_name
        assert str(exception.value) == "TestExecution resource has not been created."

        with pytest.raises(RuntimeError) as exception:
            execution.assign_input_artifacts(artifacts=[_TEST_ARTIFACT_NAME])
        assert str(exception.value) == "TestExecution resource has not been created."

        with pytest.raises(RuntimeError) as exception:
            execution.assign_output_artifacts(artifacts=[_TEST_ARTIFACT_NAME])
        assert str(exception.value) == "TestExecution resource has not been created."

        with pytest.raises(RuntimeError) as exception:
            execution.get_input_artifacts()
        assert str(exception.value) == "TestExecution resource has not been created."

        with pytest.raises(RuntimeError) as exception:
            execution.get_output_artifacts()
        assert str(exception.value) == "TestExecution resource has not been created."

        with pytest.raises(RuntimeError) as exception:
            execution.update(
                state=_TEST_EXECUTION_STATE,
                description=_TEST_DESCRIPTION,
                metadata=_TEST_UPDATED_METADATA,
            )
        assert str(exception.value) == "TestExecution resource has not been created."

    def test_base_class_parameters_overrides_default_values(self):
        class TestExecution(base_execution.BaseExecutionSchema):
            schema_title = _TEST_SCHEMA_TITLE

        execution = TestExecution(
            state=_TEST_EXECUTION_STATE,
            schema_version=_TEST_SCHEMA_VERSION,
            execution_id=_TEST_EXECUTION_ID,
            display_name=_TEST_DISPLAY_NAME,
            description=_TEST_DESCRIPTION,
            metadata=_TEST_UPDATED_METADATA,
        )
        assert execution.state == _TEST_EXECUTION_STATE
        assert execution.schema_version == _TEST_SCHEMA_VERSION
        assert execution.execution_id == _TEST_EXECUTION_ID
        assert execution.schema_title == _TEST_SCHEMA_TITLE
        assert execution.display_name == _TEST_DISPLAY_NAME
        assert execution.description == _TEST_DESCRIPTION
        assert execution.metadata == _TEST_UPDATED_METADATA

    def test_base_class_without_schema_title_raises_error(self):
        with pytest.raises(TypeError):
            base_execution.BaseExecutionSchema()

    @pytest.mark.usefixtures("create_execution_mock", "get_execution_mock")
    def test_create_method_calls_gapic_library_with_correct_parameters(
        self, create_execution_mock
    ):
        aiplatform.init(project=_TEST_PROJECT)

        class TestExecution(base_execution.BaseExecutionSchema):
            schema_title = _TEST_SCHEMA_TITLE

        execution = TestExecution(
            state=_TEST_EXECUTION_STATE,
            display_name=_TEST_DISPLAY_NAME,
            description=_TEST_DESCRIPTION,
            metadata=_TEST_UPDATED_METADATA,
        )
        execution.create(metadata_store_id=_TEST_METADATA_STORE)
        create_execution_mock.assert_called_once_with(
            parent=f"{_TEST_PARENT}/metadataStores/{_TEST_METADATA_STORE}",
            execution=mock.ANY,
            execution_id=None,
        )
        _, _, kwargs = create_execution_mock.mock_calls[0]
        assert kwargs["execution"].schema_title == _TEST_SCHEMA_TITLE
        assert kwargs["execution"].state == _TEST_EXECUTION_STATE
        assert kwargs["execution"].display_name == _TEST_DISPLAY_NAME
        assert kwargs["execution"].description == _TEST_DESCRIPTION
        assert kwargs["execution"].metadata == _TEST_UPDATED_METADATA

    @pytest.mark.usefixtures(
        "base_execution_init_with_resouce_name_mock",
        "initializer_create_client_mock",
        "create_execution_mock",
        "get_execution_mock",
    )
    def test_execution_create_call_sets_the_user_agent_header(
        self, initializer_create_client_mock
    ):
        aiplatform.init(project=_TEST_PROJECT)

        class TestExecution(base_execution.BaseExecutionSchema):
            schema_title = _TEST_SCHEMA_TITLE

        execution = TestExecution(
            state=_TEST_EXECUTION_STATE,
            display_name=_TEST_DISPLAY_NAME,
            description=_TEST_DESCRIPTION,
            metadata=_TEST_UPDATED_METADATA,
        )
        execution.create(metadata_store_id=_TEST_METADATA_STORE)
        _, _, kwargs = initializer_create_client_mock.mock_calls[0]
        assert kwargs["appended_user_agent"] == [
            "sdk_command/aiplatform.metadata.schema.base_execution.BaseExecutionSchema.create"
        ]

    @pytest.mark.usefixtures(
        "base_execution_init_with_resouce_name_mock",
        "initializer_create_client_mock",
        "create_execution_mock",
        "get_execution_mock",
    )
    def test_execution_start_execution_call_sets_the_user_agent_header(
        self, initializer_create_client_mock
    ):
        aiplatform.init(project=_TEST_PROJECT)

        class TestExecution(base_execution.BaseExecutionSchema):
            schema_title = _TEST_SCHEMA_TITLE

        execution = TestExecution(
            state=_TEST_EXECUTION_STATE,
            display_name=_TEST_DISPLAY_NAME,
            description=_TEST_DESCRIPTION,
            metadata=_TEST_UPDATED_METADATA,
        )
        execution.start_execution()
        _, _, kwargs = initializer_create_client_mock.mock_calls[0]
        assert kwargs["appended_user_agent"] == [
            "sdk_command/aiplatform.metadata.schema.base_execution.BaseExecutionSchema.start_execution"
        ]

    @pytest.mark.usefixtures(
        "initializer_create_client_mock",
        "create_execution_mock",
        "get_execution_mock",
    )
    def test_execution_init_call_sets_the_user_agent_header(
        self, initializer_create_client_mock
    ):
        aiplatform.init(project=_TEST_PROJECT)

        class TestExecution(base_execution.BaseExecutionSchema):
            schema_title = _TEST_SCHEMA_TITLE

        execution = TestExecution(
            state=_TEST_EXECUTION_STATE,
            display_name=_TEST_DISPLAY_NAME,
            description=_TEST_DESCRIPTION,
            metadata=_TEST_UPDATED_METADATA,
        )
        execution._init_with_resource_name(execution_name=_TEST_EXECUTION_NAME)
        _, _, kwargs = initializer_create_client_mock.mock_calls[0]
        assert kwargs["appended_user_agent"] == [
            "sdk_command/aiplatform.metadata.schema.base_execution.BaseExecutionSchema._init_with_resource_name"
        ]

    def test_list_executions(self, list_executions_mock):
        aiplatform.init(project=_TEST_PROJECT, location=_TEST_LOCATION)

        class TestExecution(base_execution.BaseExecutionSchema):
            schema_title = _TEST_SCHEMA_TITLE

        TestExecution.list()
        list_executions_mock.assert_called_once_with(
            request={
                "parent": f"{_TEST_PARENT}/metadataStores/default",
                "filter": f'schema_title="{_TEST_SCHEMA_TITLE}"',
            }
        )


@pytest.mark.usefixtures("google_auth_mock")
class TestMetadataBaseContextSchema:
    def setup_method(self):
        reload(initializer)
        reload(metadata)
        reload(aiplatform)

    def teardown_method(self):
        initializer.global_pool.shutdown(wait=True)

    def test_base_context_class_instatiated_uses_schema_title(self):
        class TestContext(base_context.BaseContextSchema):
            schema_title = _TEST_SCHEMA_TITLE

        context = TestContext()
        assert context.schema_title == _TEST_SCHEMA_TITLE

    def test_base_class_print_output(self, capsys):
        class TestContext(base_context.BaseContextSchema):
            schema_title = _TEST_SCHEMA_TITLE

        context = TestContext()
        print(context)
        captured = capsys.readouterr()
        assert (
            captured.out
            == f"{object.__repr__(context)}\n" + f"schema_title: {_TEST_SCHEMA_TITLE}\n"
        )

    def test_base_class_inherited_methods_error(self):
        class TestContext(base_context.BaseContextSchema):
            schema_title = _TEST_SCHEMA_TITLE

        context = TestContext()

        with pytest.raises(RuntimeError) as exception:
            context.resource_name
        assert str(exception.value) == "TestContext resource has not been created."

        with pytest.raises(RuntimeError) as exception:
            context.add_artifacts_and_executions(
                artifact_resource_names=[_TEST_ARTIFACT_NAME],
            )
        assert str(exception.value) == "TestContext resource has not been created."

        with pytest.raises(RuntimeError) as exception:
            context.get_artifacts()
        assert str(exception.value) == "TestContext resource has not been created."

        with pytest.raises(RuntimeError) as exception:
            context.add_context_children(
                contexts=[_TEST_CONTEXT_NAME],
            )
        assert str(exception.value) == "TestContext resource has not been created."

        with pytest.raises(RuntimeError) as exception:
            context.query_lineage_subgraph()
        assert str(exception.value) == "TestContext resource has not been created."

        with pytest.raises(RuntimeError) as exception:
            context.get_executions()
        assert str(exception.value) == "TestContext resource has not been created."

    def test_base_context_class_parameters_overrides_default_values(self):
        class TestContext(base_context.BaseContextSchema):
            schema_title = _TEST_SCHEMA_TITLE

        context = TestContext(
            schema_version=_TEST_SCHEMA_VERSION,
            context_id=_TEST_CONTEXT_ID,
            display_name=_TEST_DISPLAY_NAME,
            description=_TEST_DESCRIPTION,
            metadata=_TEST_UPDATED_METADATA,
        )
        assert context.schema_version == _TEST_SCHEMA_VERSION
        assert context.context_id == _TEST_CONTEXT_ID
        assert context.schema_title == _TEST_SCHEMA_TITLE
        assert context.display_name == _TEST_DISPLAY_NAME
        assert context.description == _TEST_DESCRIPTION
        assert context.metadata == _TEST_UPDATED_METADATA

    def test_base_context_class_without_schema_title_raises_error(self):
        with pytest.raises(TypeError):
            base_context.BaseContextSchema()

    @pytest.mark.usefixtures("create_context_mock", "get_context_mock")
    def test_base_context_create_is_called_with_default_parameters(
        self, create_context_mock
    ):
        aiplatform.init(project=_TEST_PROJECT)

        class TestContext(base_context.BaseContextSchema):
            schema_title = _TEST_SCHEMA_TITLE

        context = TestContext(
            schema_version=_TEST_SCHEMA_VERSION,
            context_id=_TEST_CONTEXT_ID,
            display_name=_TEST_DISPLAY_NAME,
            description=_TEST_DESCRIPTION,
            metadata=_TEST_UPDATED_METADATA,
        )
        context.create(metadata_store_id=_TEST_METADATA_STORE)
        create_context_mock.assert_called_once_with(
            parent=f"{_TEST_PARENT}/metadataStores/{_TEST_METADATA_STORE}",
            context=mock.ANY,
            context_id=_TEST_CONTEXT_ID,
        )
        _, _, kwargs = create_context_mock.mock_calls[0]
        assert kwargs["context"].schema_title == _TEST_SCHEMA_TITLE
        assert kwargs["context"].display_name == _TEST_DISPLAY_NAME
        assert kwargs["context"].description == _TEST_DESCRIPTION
        assert kwargs["context"].metadata == _TEST_UPDATED_METADATA

    @pytest.mark.usefixtures(
        "base_context_init_with_resouce_name_mock",
        "initializer_create_client_mock",
        "create_context_mock",
        "get_context_mock",
    )
    def test_base_context_create_call_sets_the_user_agent_header(
        self, initializer_create_client_mock
    ):
        aiplatform.init(project=_TEST_PROJECT)

        class TestContext(base_context.BaseContextSchema):
            schema_title = _TEST_SCHEMA_TITLE

        context = TestContext(
            schema_version=_TEST_SCHEMA_VERSION,
            context_id=_TEST_CONTEXT_ID,
            display_name=_TEST_DISPLAY_NAME,
            description=_TEST_DESCRIPTION,
            metadata=_TEST_UPDATED_METADATA,
        )
        context.create()
        _, _, kwargs = initializer_create_client_mock.mock_calls[0]
        assert kwargs["appended_user_agent"] == [
            "sdk_command/aiplatform.metadata.schema.base_context.BaseContextSchema.create"
        ]

    @pytest.mark.usefixtures(
        "initializer_create_client_mock",
        "create_context_mock",
        "get_context_mock",
    )
    def test_base_context_init_call_sets_the_user_agent_header(
        self, initializer_create_client_mock
    ):
        aiplatform.init(project=_TEST_PROJECT)

        class TestContext(base_context.BaseContextSchema):
            schema_title = _TEST_SCHEMA_TITLE

        context = TestContext(
            schema_version=_TEST_SCHEMA_VERSION,
            context_id=_TEST_CONTEXT_ID,
            display_name=_TEST_DISPLAY_NAME,
            description=_TEST_DESCRIPTION,
            metadata=_TEST_UPDATED_METADATA,
        )
        context._init_with_resource_name(context_name=_TEST_CONTEXT_NAME)
        _, _, kwargs = initializer_create_client_mock.mock_calls[0]
        assert kwargs["appended_user_agent"] == [
            "sdk_command/aiplatform.metadata.schema.base_context.BaseContextSchema._init_with_resource_name"
        ]

    def test_list_contexts(self, list_contexts_mock):
        aiplatform.init(project=_TEST_PROJECT, location=_TEST_LOCATION)

        class TestContext(base_context.BaseContextSchema):
            schema_title = _TEST_SCHEMA_TITLE

        TestContext.list()
        list_contexts_mock.assert_called_once_with(
            request={
                "parent": f"{_TEST_PARENT}/metadataStores/default",
                "filter": f'schema_title="{_TEST_SCHEMA_TITLE}"',
            }
        )


@pytest.mark.usefixtures("google_auth_mock")
class TestMetadataGoogleArtifactSchema:
    def setup_method(self):
        reload(initializer)
        reload(metadata)
        reload(aiplatform)

    def teardown_method(self):
        initializer.global_pool.shutdown(wait=True)

    def test_vertex_dataset_schema_title_is_set_correctly(self):
        artifact = google_artifact_schema.VertexDataset(
            vertex_dataset_name=_TEST_ARTIFACT_NAME,
        )
        assert artifact.schema_title == "google.VertexDataset"

    def test_vertex_dataset_constructor_parameters_are_set_correctly(self):
        artifact = google_artifact_schema.VertexDataset(
            vertex_dataset_name=f"{_TEST_PARENT}/datasets/dataset-id",
            display_name=_TEST_DISPLAY_NAME,
            schema_version=_TEST_SCHEMA_VERSION,
            description=_TEST_DESCRIPTION,
            metadata={},
        )
        assert (
            artifact.uri
            == "https://us-central1-aiplatform.googleapis.com/v1/projects/test-project/locations/us-central1/datasets/dataset-id"
        )
        assert artifact.display_name == _TEST_DISPLAY_NAME
        assert artifact.description == _TEST_DESCRIPTION
        assert artifact.metadata == {
            "resourceName": "projects/test-project/locations/us-central1/datasets/dataset-id"
        }
        assert artifact.schema_version == _TEST_SCHEMA_VERSION

    def test_vertex_model_schema_title_is_set_correctly(self):
        artifact = google_artifact_schema.VertexModel(
            vertex_model_name=_TEST_ARTIFACT_NAME,
        )
        assert artifact.schema_title == "google.VertexModel"

    def test_vertex_model_constructor_parameters_are_set_correctly(self):
        artifact = google_artifact_schema.VertexModel(
            vertex_model_name=f"{_TEST_PARENT}/models/model-id",
            display_name=_TEST_DISPLAY_NAME,
            schema_version=_TEST_SCHEMA_VERSION,
            description=_TEST_DESCRIPTION,
            metadata={},
        )
        assert (
            artifact.uri
            == "https://us-central1-aiplatform.googleapis.com/v1/projects/test-project/locations/us-central1/models/model-id"
        )
        assert artifact.display_name == _TEST_DISPLAY_NAME
        assert artifact.description == _TEST_DESCRIPTION
        assert artifact.metadata == {
            "resourceName": "projects/test-project/locations/us-central1/models/model-id"
        }
        assert artifact.schema_version == _TEST_SCHEMA_VERSION

    def test_vertex_endpoint_schema_title_is_set_correctly(self):
        artifact = google_artifact_schema.VertexEndpoint(
            vertex_endpoint_name=_TEST_ARTIFACT_NAME,
        )
        assert artifact.schema_title == "google.VertexEndpoint"

    def test_vertex_endpoint_constructor_parameters_are_set_correctly(self):
        artifact = google_artifact_schema.VertexEndpoint(
            vertex_endpoint_name=f"{_TEST_PARENT}/endpoints/endpoint-id",
            display_name=_TEST_DISPLAY_NAME,
            schema_version=_TEST_SCHEMA_VERSION,
            description=_TEST_DESCRIPTION,
            metadata={},
        )
        assert (
            artifact.uri
            == "https://us-central1-aiplatform.googleapis.com/v1/projects/test-project/locations/us-central1/endpoints/endpoint-id"
        )
        assert artifact.display_name == _TEST_DISPLAY_NAME
        assert artifact.description == _TEST_DESCRIPTION
        assert artifact.metadata == {
            "resourceName": "projects/test-project/locations/us-central1/endpoints/endpoint-id"
        }
        assert artifact.schema_version == _TEST_SCHEMA_VERSION

    def test_unmanaged_container_model_title_is_set_correctly(self):
        predict_schemata = utils.PredictSchemata(
            instance_schema_uri="instance_uri",
            prediction_schema_uri="prediction_uri",
            parameters_schema_uri="parameters_uri",
        )

        container_spec = utils.ContainerSpec(
            image_uri="gcr.io/test_container_image_uri"
        )
        artifact = google_artifact_schema.UnmanagedContainerModel(
            predict_schemata=predict_schemata,
            container_spec=container_spec,
        )
        assert artifact.schema_title == "google.UnmanagedContainerModel"

    def test_unmanaged_container_model_constructor_parameters_are_set_correctly(self):
        predict_schemata = utils.PredictSchemata(
            instance_schema_uri="instance_uri",
            prediction_schema_uri="prediction_uri",
            parameters_schema_uri="parameters_uri",
        )

        container_spec = utils.ContainerSpec(
            image_uri="gcr.io/test_container_image_uri"
        )

        artifact = google_artifact_schema.UnmanagedContainerModel(
            predict_schemata=predict_schemata,
            container_spec=container_spec,
            artifact_id=_TEST_ARTIFACT_ID,
            uri=_TEST_URI,
            display_name=_TEST_DISPLAY_NAME,
            schema_version=_TEST_SCHEMA_VERSION,
            description=_TEST_DESCRIPTION,
            metadata=_TEST_UPDATED_METADATA,
        )
        expected_metadata = {
            "test-param1": 2.0,
            "test-param2": "test-value-1",
            "test-param3": False,
            "predictSchemata": {
                "instanceSchemaUri": "instance_uri",
                "parametersSchemaUri": "parameters_uri",
                "predictionSchemaUri": "prediction_uri",
            },
            "containerSpec": {"imageUri": "gcr.io/test_container_image_uri"},
        }

        assert artifact.artifact_id == _TEST_ARTIFACT_ID
        assert artifact.uri == _TEST_URI
        assert artifact.display_name == _TEST_DISPLAY_NAME
        assert artifact.description == _TEST_DESCRIPTION
        assert json.dumps(artifact.metadata, sort_keys=True) == json.dumps(
            expected_metadata, sort_keys=True
        )
        assert artifact.schema_version == _TEST_SCHEMA_VERSION

    def test_classification_metrics_title_is_set_correctly(self):
        artifact = google_artifact_schema.ClassificationMetrics()
        assert artifact.schema_title == "google.ClassificationMetrics"

    def test_classification_metrics_constructor_parameters_are_set_correctly(self):
        aggregation_type = "MACRO_AVERAGE"
        aggregation_threshold = 0.5
        recall = 0.5
        precision = 0.5
        f1_score = 0.5
        accuracy = 0.5
        au_prc = 1.0
        au_roc = 2.0
        log_loss = 0.5
        confusion_matrix = utils.ConfusionMatrix(
            matrix=[[9.0, 1.0], [1.0, 9.0]],
            annotation_specs=[
                utils.AnnotationSpec(display_name="cat"),
                utils.AnnotationSpec(display_name="dog"),
            ],
        )
        confidence_metrics = [
            utils.ConfidenceMetric(
                confidence_threshold=0.9, recall=0.1, false_positive_rate=0.1
            ),
            utils.ConfidenceMetric(
                confidence_threshold=0.5, recall=0.5, false_positive_rate=0.7
            ),
            utils.ConfidenceMetric(
                confidence_threshold=0.1, recall=0.9, false_positive_rate=0.9
            ),
        ]

        artifact = google_artifact_schema.ClassificationMetrics(
            aggregation_type=aggregation_type,
            aggregation_threshold=aggregation_threshold,
            recall=recall,
            precision=precision,
            f1_score=f1_score,
            accuracy=accuracy,
            au_prc=au_prc,
            au_roc=au_roc,
            log_loss=log_loss,
            confusion_matrix=confusion_matrix,
            confidence_metrics=confidence_metrics,
            artifact_id=_TEST_ARTIFACT_ID,
            uri=_TEST_URI,
            display_name=_TEST_DISPLAY_NAME,
            schema_version=_TEST_SCHEMA_VERSION,
            description=_TEST_DESCRIPTION,
            metadata=_TEST_UPDATED_METADATA,
        )
        expected_metadata = {
            "test-param1": _TEST_UPDATED_METADATA["test-param1"],
            "test-param2": _TEST_UPDATED_METADATA["test-param2"],
            "test-param3": _TEST_UPDATED_METADATA["test-param3"],
            "aggregationType": aggregation_type,
            "aggregationThreshold": aggregation_threshold,
            "recall": recall,
            "precision": precision,
            "f1Score": f1_score,
            "accuracy": accuracy,
            "auPrc": au_prc,
            "auRoc": au_roc,
            "logLoss": log_loss,
            "confusionMatrix": confusion_matrix.to_dict(),
            "confidenceMetrics": [
                confidence_metric.to_dict() for confidence_metric in confidence_metrics
            ],
        }

        assert artifact.artifact_id == _TEST_ARTIFACT_ID
        assert artifact.uri == _TEST_URI
        assert artifact.display_name == _TEST_DISPLAY_NAME
        assert artifact.description == _TEST_DESCRIPTION
        assert json.dumps(artifact.metadata, sort_keys=True) == json.dumps(
            expected_metadata, sort_keys=True
        )
        assert artifact.schema_version == _TEST_SCHEMA_VERSION

    def test_classification_metrics_wrong_aggregation_type(self):
        with pytest.raises(ValueError) as exception:
            google_artifact_schema.ClassificationMetrics(
                aggregation_type="unspecified_type"
            )
        assert (
            str(exception.value)
            == "aggregation_type can only be 'AGGREGATION_TYPE_UNSPECIFIED', 'MACRO_AVERAGE', or 'MICRO_AVERAGE'."
        )

    def test_regression_metrics_title_is_set_correctly(self):
        artifact = google_artifact_schema.RegressionMetrics()
        assert artifact.schema_title == "google.RegressionMetrics"

    def test_regression_metrics_constructor_parameters_are_set_correctly(self):
        root_mean_squared_error = 1.0
        mean_absolute_error = 2.0
        mean_absolute_percentage_error = 0.2
        r_squared = 0.5
        root_mean_squared_log_error = 0.9

        artifact = google_artifact_schema.RegressionMetrics(
            root_mean_squared_error=root_mean_squared_error,
            mean_absolute_error=mean_absolute_error,
            mean_absolute_percentage_error=mean_absolute_percentage_error,
            r_squared=r_squared,
            root_mean_squared_log_error=root_mean_squared_log_error,
            artifact_id=_TEST_ARTIFACT_ID,
            uri=_TEST_URI,
            display_name=_TEST_DISPLAY_NAME,
            schema_version=_TEST_SCHEMA_VERSION,
            description=_TEST_DESCRIPTION,
            metadata=_TEST_UPDATED_METADATA,
        )
        expected_metadata = {
            "test-param1": 2.0,
            "test-param2": "test-value-1",
            "test-param3": False,
            "rootMeanSquaredError": 1.0,
            "meanAbsoluteError": 2.0,
            "meanAbsolutePercentageError": 0.2,
            "rSquared": 0.5,
            "rootMeanSquaredLogError": 0.9,
        }

        assert artifact.artifact_id == _TEST_ARTIFACT_ID
        assert artifact.uri == _TEST_URI
        assert artifact.display_name == _TEST_DISPLAY_NAME
        assert artifact.description == _TEST_DESCRIPTION
        assert json.dumps(artifact.metadata, sort_keys=True) == json.dumps(
            expected_metadata, sort_keys=True
        )
        assert artifact.schema_version == _TEST_SCHEMA_VERSION

    def test_forecasting_metrics_title_is_set_correctly(self):
        artifact = google_artifact_schema.ForecastingMetrics()
        assert artifact.schema_title == "google.ForecastingMetrics"

    def test_forecasting_metrics_constructor_parameters_are_set_correctly(self):
        root_mean_squared_error = 1.0
        mean_absolute_error = 2.0
        mean_absolute_percentage_error = 0.2
        r_squared = 0.5
        root_mean_squared_log_error = 0.9
        weighted_absolute_percentage_error = 4.0
        root_mean_squared_percentage_error = 0.7
        symmetric_mean_absolute_percentage_error = 0.8

        artifact = google_artifact_schema.ForecastingMetrics(
            root_mean_squared_error=root_mean_squared_error,
            mean_absolute_error=mean_absolute_error,
            mean_absolute_percentage_error=mean_absolute_percentage_error,
            r_squared=r_squared,
            root_mean_squared_log_error=root_mean_squared_log_error,
            weighted_absolute_percentage_error=weighted_absolute_percentage_error,
            root_mean_squared_percentage_error=root_mean_squared_percentage_error,
            symmetric_mean_absolute_percentage_error=symmetric_mean_absolute_percentage_error,
            artifact_id=_TEST_ARTIFACT_ID,
            uri=_TEST_URI,
            display_name=_TEST_DISPLAY_NAME,
            schema_version=_TEST_SCHEMA_VERSION,
            description=_TEST_DESCRIPTION,
            metadata=_TEST_UPDATED_METADATA,
        )
        expected_metadata = {
            "test-param1": 2.0,
            "test-param2": "test-value-1",
            "test-param3": False,
            "rootMeanSquaredError": 1.0,
            "meanAbsoluteError": 2.0,
            "meanAbsolutePercentageError": 0.2,
            "rSquared": 0.5,
            "rootMeanSquaredLogError": 0.9,
            "weightedAbsolutePercentageError": 4.0,
            "rootMeanSquaredPercentageError": 0.7,
            "symmetricMeanAbsolutePercentageError": 0.8,
        }

        assert artifact.artifact_id == _TEST_ARTIFACT_ID
        assert artifact.uri == _TEST_URI
        assert artifact.display_name == _TEST_DISPLAY_NAME
        assert artifact.description == _TEST_DESCRIPTION
        assert json.dumps(artifact.metadata, sort_keys=True) == json.dumps(
            expected_metadata, sort_keys=True
        )
        assert artifact.schema_version == _TEST_SCHEMA_VERSION

    def test_experiment_model_title_is_set_correctly(self):
        artifact = google_artifact_schema.ExperimentModel(
            framework_name="sklearn",
            framework_version="1.0.0",
            model_file="model.pkl",
            uri=_TEST_URI,
        )
        assert artifact.schema_title == "google.ExperimentModel"
        assert artifact.framework_name == "sklearn"
        assert artifact.framework_version == "1.0.0"
        assert artifact.uri == _TEST_URI

    def test_experiment_model_wrong_metadata_key(self):
        with pytest.raises(ValueError) as exception:
            google_artifact_schema.ExperimentModel(
                framework_name="sklearn",
                framework_version="1.0.0",
                model_file="model.pkl",
                uri=_TEST_URI,
                metadata={"modelFile": "abc"},
            )
        assert (
            str(exception.value) == "'modelFile' is a system reserved key in metadata."
        )

    def test_experiment_model_constructor_parameters_are_set_correctly(self):
        predict_schemata = utils.PredictSchemata(
            instance_schema_uri="instance_uri",
            prediction_schema_uri="prediction_uri",
            parameters_schema_uri="parameters_uri",
        )

        artifact = google_artifact_schema.ExperimentModel(
            framework_name="sklearn",
            framework_version="1.0.0",
            model_file="model.pkl",
            model_class="sklearn.linear_model._base.LinearRegression",
            predict_schemata=predict_schemata,
            artifact_id=_TEST_ARTIFACT_ID,
            uri=_TEST_URI,
            display_name=_TEST_DISPLAY_NAME,
            schema_version=_TEST_SCHEMA_VERSION,
            description=_TEST_DESCRIPTION,
            metadata=_TEST_UPDATED_METADATA,
        )
        expected_metadata = {
            "test-param1": 2.0,
            "test-param2": "test-value-1",
            "test-param3": False,
            "frameworkName": "sklearn",
            "frameworkVersion": "1.0.0",
            "modelFile": "model.pkl",
            "modelClass": "sklearn.linear_model._base.LinearRegression",
            "predictSchemata": {
                "instanceSchemaUri": "instance_uri",
                "parametersSchemaUri": "parameters_uri",
                "predictionSchemaUri": "prediction_uri",
            },
        }

        assert artifact.artifact_id == _TEST_ARTIFACT_ID
        assert artifact.uri == _TEST_URI
        assert artifact.display_name == _TEST_DISPLAY_NAME
        assert artifact.description == _TEST_DESCRIPTION
        assert json.dumps(artifact.metadata, sort_keys=True) == json.dumps(
            expected_metadata, sort_keys=True
        )
        assert artifact.schema_version == _TEST_SCHEMA_VERSION


@pytest.mark.usefixtures("google_auth_mock")
class TestMetadataSystemArtifactSchema:
    def setup_method(self):
        reload(initializer)
        reload(metadata)
        reload(aiplatform)

    def teardown_method(self):
        initializer.global_pool.shutdown(wait=True)

    def test_system_dataset_schema_title_is_set_correctly(self):
        artifact = system_artifact_schema.Dataset()
        assert artifact.schema_title == "system.Dataset"

    def test_system_dataset_constructor_parameters_are_set_correctly(self):
        artifact = system_artifact_schema.Dataset(
            uri=_TEST_URI,
            artifact_id=_TEST_ARTIFACT_ID,
            display_name=_TEST_DISPLAY_NAME,
            schema_version=_TEST_SCHEMA_VERSION,
            description=_TEST_DESCRIPTION,
            metadata=_TEST_UPDATED_METADATA,
        )
        assert artifact.uri == _TEST_URI
        assert artifact.artifact_id == _TEST_ARTIFACT_ID
        assert artifact.display_name == _TEST_DISPLAY_NAME
        assert artifact.description == _TEST_DESCRIPTION
        assert artifact.metadata == _TEST_UPDATED_METADATA
        assert artifact.schema_version == _TEST_SCHEMA_VERSION

    def test_system_artifact_schema_title_is_set_correctly(self):
        artifact = system_artifact_schema.Artifact()
        assert artifact.schema_title == "system.Artifact"

    def test_system_artifact_constructor_parameters_are_set_correctly(self):
        artifact = system_artifact_schema.Artifact(
            uri=_TEST_URI,
            artifact_id=_TEST_ARTIFACT_ID,
            display_name=_TEST_DISPLAY_NAME,
            schema_version=_TEST_SCHEMA_VERSION,
            description=_TEST_DESCRIPTION,
            metadata=_TEST_UPDATED_METADATA,
        )
        assert artifact.uri == _TEST_URI
        assert artifact.artifact_id == _TEST_ARTIFACT_ID
        assert artifact.display_name == _TEST_DISPLAY_NAME
        assert artifact.description == _TEST_DESCRIPTION
        assert artifact.metadata == _TEST_UPDATED_METADATA
        assert artifact.schema_version == _TEST_SCHEMA_VERSION

    def test_system_model_schema_title_is_set_correctly(self):
        artifact = system_artifact_schema.Model()
        assert artifact.schema_title == "system.Model"

    def test_system_model_constructor_parameters_are_set_correctly(self):
        artifact = system_artifact_schema.Model(
            uri=_TEST_URI,
            artifact_id=_TEST_ARTIFACT_ID,
            display_name=_TEST_DISPLAY_NAME,
            schema_version=_TEST_SCHEMA_VERSION,
            description=_TEST_DESCRIPTION,
            metadata=_TEST_UPDATED_METADATA,
        )
        assert artifact.uri == _TEST_URI
        assert artifact.artifact_id == _TEST_ARTIFACT_ID
        assert artifact.display_name == _TEST_DISPLAY_NAME
        assert artifact.description == _TEST_DESCRIPTION
        assert artifact.metadata == _TEST_UPDATED_METADATA
        assert artifact.schema_version == _TEST_SCHEMA_VERSION

    def test_system_metrics_schema_title_is_set_correctly(self):
        artifact = system_artifact_schema.Metrics()
        assert artifact.schema_title == "system.Metrics"

    def test_system_metrics_values_default_to_none(self):
        artifact = system_artifact_schema.Metrics()
        assert artifact._gca_resource.metadata is None

    def test_system_metrics_constructor_parameters_are_set_correctly(self):
        artifact = system_artifact_schema.Metrics(
            accuracy=0.1,
            precision=0.2,
            recall=0.3,
            f1score=0.4,
            mean_absolute_error=0.5,
            mean_squared_error=0.6,
            artifact_id=_TEST_ARTIFACT_ID,
            uri=_TEST_URI,
            display_name=_TEST_DISPLAY_NAME,
            schema_version=_TEST_SCHEMA_VERSION,
            description=_TEST_DESCRIPTION,
            metadata=_TEST_UPDATED_METADATA,
        )
        assert artifact.uri == _TEST_URI
        assert artifact.artifact_id == _TEST_ARTIFACT_ID
        assert artifact.display_name == _TEST_DISPLAY_NAME
        assert artifact.description == _TEST_DESCRIPTION
        assert artifact.schema_version == _TEST_SCHEMA_VERSION
        assert artifact.metadata["accuracy"] == 0.1
        assert artifact.metadata["precision"] == 0.2
        assert artifact.metadata["recall"] == 0.3
        assert artifact.metadata["f1score"] == 0.4
        assert artifact.metadata["mean_absolute_error"] == 0.5
        assert artifact.metadata["mean_squared_error"] == 0.6


@pytest.mark.usefixtures("google_auth_mock")
class TestMetadataSystemSchemaExecution:
    def setup_method(self):
        reload(initializer)
        reload(metadata)
        reload(aiplatform)

    def teardown_method(self):
        initializer.global_pool.shutdown(wait=True)

    # Test system.Execution Schemas
    def test_system_container_execution_schema_title_is_set_correctly(self):
        execution = system_execution_schema.ContainerExecution()
        assert execution.schema_title == "system.ContainerExecution"

    def test_system_custom_job_execution_schema_title_is_set_correctly(self):
        execution = system_execution_schema.CustomJobExecution()
        assert execution.schema_title == "system.CustomJobExecution"

    def test_system_run_execution_schema_title_is_set_correctly(self):
        execution = system_execution_schema.Run()
        assert execution.schema_title == "system.Run"


@pytest.mark.usefixtures("google_auth_mock")
class TestMetadataSystemSchemaContext:
    def setup_method(self):
        reload(initializer)
        reload(metadata)
        reload(aiplatform)

    def teardown_method(self):
        initializer.global_pool.shutdown(wait=True)

    # Test system.Context Schemas
    @pytest.mark.usefixtures("create_context_mock", "get_context_mock")
    def test_create_is_called_with_default_parameters(self, create_context_mock):
        aiplatform.init(project=_TEST_PROJECT)

        class TestContext(base_context.BaseContextSchema):
            schema_title = _TEST_SCHEMA_TITLE

        context = TestContext(
            display_name=_TEST_DISPLAY_NAME,
            description=_TEST_DESCRIPTION,
            metadata=_TEST_UPDATED_METADATA,
        )
        context.create(metadata_store_id=_TEST_METADATA_STORE)
        create_context_mock.assert_called_once_with(
            parent=f"{_TEST_PARENT}/metadataStores/{_TEST_METADATA_STORE}",
            context=mock.ANY,
            context_id=None,
        )
        _, _, kwargs = create_context_mock.mock_calls[0]
        assert kwargs["context"].schema_title == _TEST_SCHEMA_TITLE
        assert kwargs["context"].display_name == _TEST_DISPLAY_NAME
        assert kwargs["context"].description == _TEST_DESCRIPTION
        assert kwargs["context"].metadata == _TEST_UPDATED_METADATA

    def test_system_experiment_schema_title_is_set_correctly(self):
        context = system_context_schema.Experiment()
        assert context.schema_title == "system.Experiment"

    def test_system_experiment_run_schema_title_is_set_correctly(self):
        context = system_context_schema.ExperimentRun()
        assert context.schema_title == "system.ExperimentRun"

    def test_system_experiment_run_parameters_are_set_correctly(self):
        context = system_context_schema.ExperimentRun(experiment_id=_TEST_CONTEXT_ID)
        assert context.metadata["experiment_id"] == _TEST_CONTEXT_ID

    def test_system_pipeline_schema_title_is_set_correctly(self):
        context = system_context_schema.Pipeline()
        assert context.schema_title == "system.Pipeline"

    def test_system_pipeline_run_schema_title_is_set_correctly(self):
        context = system_context_schema.PipelineRun()
        assert context.schema_title == "system.PipelineRun"

    def test_system_pipeline_run_parameters_are_set_correctly(self):
        context = system_context_schema.PipelineRun(pipeline_id=_TEST_CONTEXT_ID)
        assert context.metadata["pipeline_id"] == _TEST_CONTEXT_ID


@pytest.mark.usefixtures("google_auth_mock")
class TestMetadataUtils:
    def setup_method(self):
        reload(initializer)
        reload(metadata)
        reload(aiplatform)

    def teardown_method(self):
        initializer.global_pool.shutdown(wait=True)

    def test_predict_schemata_to_dict_method_returns_correct_schema(self):
        predict_schemata = utils.PredictSchemata(
            instance_schema_uri="instance_uri",
            prediction_schema_uri="prediction_uri",
            parameters_schema_uri="parameters_uri",
        )
        expected_results = {
            "instanceSchemaUri": "instance_uri",
            "parametersSchemaUri": "parameters_uri",
            "predictionSchemaUri": "prediction_uri",
        }

        assert json.dumps(predict_schemata.to_dict()) == json.dumps(expected_results)

    def test_create_uri_from_resource_name_for_valid_resouce_names(self):
        valid_resouce_names = [
            "projects/project/locations/location/resource_type/resource_id",
            "projects/project/locations/location/resource_type/resource_id@version",
            "projects/project/locations/location/metadataStores/store_id/resource_type/resource_id",
            "projects/project/locations/location/metadataStores/store_id/resource_type/resource_id@version",
        ]
        for resouce_name in valid_resouce_names:
            uri = utils.create_uri_from_resource_name(resource_name=resouce_name)
            assert (
                uri == f"https://location-aiplatform.googleapis.com/v1/{resouce_name}"
            )

    def test_create_uri_from_resource_name_for_invalid_resouce_names(self):
        invalid_resouce_name = (
            "projects/project/locations/location/resource_type/resource_id/"
        )
        with pytest.raises(ValueError):
            utils.create_uri_from_resource_name(resource_name=invalid_resouce_name)

    def test_container_spec_to_dict_method_returns_correct_schema(self):
        container_spec = utils.ContainerSpec(
            image_uri="gcr.io/some_container_image_uri",
            command=["test_command"],
            args=["test_args"],
            env=[{"env_var_name": "env_var_value"}],
            ports=[1],
            predict_route="test_prediction_rout",
            health_route="test_health_rout",
        )

        expected_results = {
            "imageUri": "gcr.io/some_container_image_uri",
            "command": ["test_command"],
            "args": ["test_args"],
            "env": [{"env_var_name": "env_var_value"}],
            "ports": [1],
            "predictRoute": "test_prediction_rout",
            "healthRoute": "test_health_rout",
        }

        assert json.dumps(container_spec.to_dict()) == json.dumps(expected_results)

    def test_annotation_spec_to_dict_method_returns_correct_schema(self):
        annotation_spec = utils.AnnotationSpec(
            display_name="test_display_name",
            id="test_annotation_id",
        )

        expected_results = {
            "displayName": "test_display_name",
            "id": "test_annotation_id",
        }

        assert json.dumps(annotation_spec.to_dict(), sort_keys=True) == json.dumps(
            expected_results, sort_keys=True
        )

    def test_confusion_matrix_to_dict_method_returns_correct_schema(self):
        confusion_matrix = utils.ConfusionMatrix(
            matrix=[[9, 1], [1, 9]],
            annotation_specs=[
                utils.AnnotationSpec(display_name="cat"),
                utils.AnnotationSpec(display_name="dog"),
            ],
        )

        expected_results = {
            "rows": [[9, 1], [1, 9]],
            "annotationSpecs": [
                {"displayName": "cat"},
                {"displayName": "dog"},
            ],
        }

        assert json.dumps(confusion_matrix.to_dict(), sort_keys=True) == json.dumps(
            expected_results, sort_keys=True
        )

    def test_confusion_matrix_to_dict_method_length_error(self):
        confusion_matrix = utils.ConfusionMatrix(
            matrix=[[9, 1], [1, 9]],
            annotation_specs=[
                utils.AnnotationSpec(display_name="cat"),
                utils.AnnotationSpec(display_name="dog"),
                utils.AnnotationSpec(display_name="bird"),
            ],
        )

        with pytest.raises(ValueError) as exception:
            confusion_matrix.to_dict()
        assert (
            str(exception.value)
            == "Length of annotation_specs and matrix must be the same. Got lengths 3 and 2 respectively."
        )

    def test_confidence_metric_to_dict_method_returns_correct_schema(self):
        confidence_metric = utils.ConfidenceMetric(
            confidence_threshold=0.5,
            recall=0.5,
            precision=0.5,
            f1_score=0.5,
            max_predictions=1,
            false_positive_rate=0.5,
            accuracy=0.5,
            true_positive_count=1,
            false_positive_count=1,
            false_negative_count=1,
            true_negative_count=1,
            recall_at_1=0.5,
            precision_at_1=0.5,
            false_positive_rate_at_1=0.5,
            f1_score_at_1=0.5,
            confusion_matrix=utils.ConfusionMatrix(
                matrix=[[9, 1], [1, 9]],
                annotation_specs=[
                    utils.AnnotationSpec(display_name="cat"),
                    utils.AnnotationSpec(display_name="dog"),
                ],
            ),
        )

        expected_results = {
            "confidenceThreshold": 0.5,
            "recall": 0.5,
            "precision": 0.5,
            "f1Score": 0.5,
            "maxPredictions": 1,
            "falsePositiveRate": 0.5,
            "accuracy": 0.5,
            "truePositiveCount": 1,
            "falsePositiveCount": 1,
            "falseNegativeCount": 1,
            "trueNegativeCount": 1,
            "recallAt1": 0.5,
            "precisionAt1": 0.5,
            "falsePositiveRateAt1": 0.5,
            "f1ScoreAt1": 0.5,
            "confusionMatrix": {
                "rows": [[9, 1], [1, 9]],
                "annotationSpecs": [
                    {"displayName": "cat"},
                    {"displayName": "dog"},
                ],
            },
        }

        assert json.dumps(confidence_metric.to_dict(), sort_keys=True) == json.dumps(
            expected_results, sort_keys=True
        )

    @pytest.mark.usefixtures("create_execution_mock", "get_execution_mock")
    def test_start_execution_method_calls_gapic_library_with_correct_parameters(
        self, create_execution_mock
    ):
        aiplatform.init(project=_TEST_PROJECT)

        class TestExecution(base_execution.BaseExecutionSchema):
            schema_title = _TEST_SCHEMA_TITLE

        execution = TestExecution(
            state=_TEST_EXECUTION_STATE,
            display_name=_TEST_DISPLAY_NAME,
            description=_TEST_DESCRIPTION,
            metadata=_TEST_UPDATED_METADATA,
        )
        execution.start_execution()
        create_execution_mock.assert_called_once_with(
            parent=f"{_TEST_PARENT}/metadataStores/default",
            execution=mock.ANY,
            execution_id=None,
        )
        _, _, kwargs = create_execution_mock.mock_calls[0]
        assert kwargs["execution"].schema_title == _TEST_SCHEMA_TITLE
        assert kwargs["execution"].display_name == _TEST_DISPLAY_NAME
        assert kwargs["execution"].description == _TEST_DESCRIPTION
        assert kwargs["execution"].metadata == _TEST_UPDATED_METADATA



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from typing import MutableMapping, MutableSequence

import proto  # type: ignore

from google.protobuf import timestamp_pb2  # type: ignore
from google.rpc import status_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.cloud.aiplatform.v1beta1",
    manifest={
        "GenericOperationMetadata",
        "DeleteOperationMetadata",
    },
)


class GenericOperationMetadata(proto.Message):
    r"""Generic Metadata shared by all operations.

    Attributes:
        partial_failures (MutableSequence[google.rpc.status_pb2.Status]):
            Output only. Partial failures encountered.
            E.g. single files that couldn't be read.
            This field should never exceed 20 entries.
            Status details field will contain standard
            Google Cloud error details.
        create_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Time when the operation was
            created.
        update_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Time when the operation was
            updated for the last time. If the operation has
            finished (successfully or not), this is the
            finish time.
    """

    partial_failures: MutableSequence[status_pb2.Status] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=status_pb2.Status,
    )
    create_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=2,
        message=timestamp_pb2.Timestamp,
    )
    update_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )


class DeleteOperationMetadata(proto.Message):
    r"""Details of operations that perform deletes of any entities.

    Attributes:
        generic_metadata (google.cloud.aiplatform_v1beta1.types.GenericOperationMetadata):
            The common part of the operation metadata.
    """

    generic_metadata: "GenericOperationMetadata" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="GenericOperationMetadata",
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-

# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import importlib
import logging
import multiprocessing
import os
import traceback

try:
    from fastapi import FastAPI
    from fastapi import HTTPException
    from fastapi import Request
    from fastapi import Response
except ImportError:
    raise ImportError(
        "FastAPI is not installed and is required to run model servers. "
        'Please install the SDK using `pip install "google-cloud-aiplatform[prediction]>=1.16.0"`.'
    )

try:
    import uvicorn
except ImportError:
    raise ImportError(
        "Uvicorn is not installed and is required to run fastapi applications. "
        'Please install the SDK using `pip install "google-cloud-aiplatform[prediction]>=1.16.0"`.'
    )

from google.cloud.aiplatform.constants import prediction
from google.cloud.aiplatform import version


class CprModelServer:
    """Model server to do custom prediction routines."""

    def __init__(self):
        """Initializes a fastapi application and sets the configs.

        Raises:
            ValueError: If either HANDLER_MODULE or HANDLER_CLASS is not set in the
                environment variables. Or if any of AIP_HTTP_PORT, AIP_HEALTH_ROUTE,
                and AIP_PREDICT_ROUTE is not set in the environment variables.
        """
        self._init_logging()

        if "HANDLER_MODULE" not in os.environ or "HANDLER_CLASS" not in os.environ:
            raise ValueError(
                "Both of the environment variables, HANDLER_MODULE and HANDLER_CLASS "
                "need to be specified."
            )
        handler_module = importlib.import_module(os.environ.get("HANDLER_MODULE"))
        handler_class = getattr(handler_module, os.environ.get("HANDLER_CLASS"))
        self.is_default_handler = (
            handler_module == "google.cloud.aiplatform.prediction.handler"
        )

        predictor_class = None
        if "PREDICTOR_MODULE" in os.environ:
            predictor_module = importlib.import_module(
                os.environ.get("PREDICTOR_MODULE")
            )
            predictor_class = getattr(
                predictor_module, os.environ.get("PREDICTOR_CLASS")
            )

        self.handler = handler_class(
            os.environ.get("AIP_STORAGE_URI"), predictor=predictor_class
        )

        if "AIP_HTTP_PORT" not in os.environ:
            raise ValueError(
                "The environment variable AIP_HTTP_PORT needs to be specified."
            )
        if (
            "AIP_HEALTH_ROUTE" not in os.environ
            or "AIP_PREDICT_ROUTE" not in os.environ
        ):
            raise ValueError(
                "Both of the environment variables AIP_HEALTH_ROUTE and "
                "AIP_PREDICT_ROUTE need to be specified."
            )
        self.http_port = int(os.environ.get("AIP_HTTP_PORT"))
        self.health_route = os.environ.get("AIP_HEALTH_ROUTE")
        self.predict_route = os.environ.get("AIP_PREDICT_ROUTE")

        self.app = FastAPI()
        self.app.add_api_route(
            path=self.health_route,
            endpoint=self.health,
            methods=["GET"],
        )
        self.app.add_api_route(
            path=self.predict_route,
            endpoint=self.predict,
            methods=["POST"],
        )

    async def __call__(self, scope, receive, send):
        await self.app(scope, receive, send)

    def _init_logging(self):
        """Initializes the logging config."""
        logging.basicConfig(
            format="%(asctime)s: %(message)s",
            datefmt="%m/%d/%Y %I:%M:%S %p",
            level=logging.INFO,
        )

    def health(self):
        """Executes a health check."""
        return {}

    async def predict(self, request: Request) -> Response:
        """Executes a prediction.

        Args:
            request (Request):
                Required. The prediction request.

        Returns:
            The response containing prediction results.

        Raises:
            HTTPException: If the handle function of the handler raises any exceptions.
        """
        try:
            return await self.handler.handle(request)
        except HTTPException:
            # Raises exception if it's a HTTPException.
            raise
        except Exception as exception:
            error_message = "An exception {} occurred. Arguments: {}.".format(
                type(exception).__name__, exception.args
            )
            logging.info(
                "{}\\nTraceback: {}".format(error_message, traceback.format_exc())
            )

            # Converts all other exceptions to HTTPException.
            if self.is_default_handler:
                raise HTTPException(
                    status_code=500,
                    detail=error_message,
                    headers={
                        prediction.CUSTOM_PREDICTION_ROUTINES_SERVER_ERROR_HEADER_KEY: version.__version__
                    },
                )
            raise HTTPException(status_code=500, detail=error_message)


def set_number_of_workers_from_env() -> None:
    """Sets the number of model server workers used by Uvicorn in the environment variable.

    The number of model server workers will be set as WEB_CONCURRENCY in the environment
    variables.
    The default number of model server workers is the number of cores.
    The following environment variables will adjust the number of workers:
        VERTEX_CPR_WEB_CONCURRENCY:
            The number of the workers. This will overwrite the number calculated by the other
            variables, min(VERTEX_CPR_WORKERS_PER_CORE * number_of_cores, VERTEX_CPR_MAX_WORKERS).
        VERTEX_CPR_WORKERS_PER_CORE:
            The number of the workers per core. The default is 1.
        VERTEX_CPR_MAX_WORKERS:
            The maximum number of workers can be used given the value of VERTEX_CPR_WORKERS_PER_CORE
            and the number of cores.
    """
    workers_per_core_str = os.getenv("VERTEX_CPR_WORKERS_PER_CORE", "1")
    max_workers_str = os.getenv("VERTEX_CPR_MAX_WORKERS")
    use_max_workers = None
    if max_workers_str:
        use_max_workers = int(max_workers_str)
    web_concurrency_str = os.getenv("VERTEX_CPR_WEB_CONCURRENCY")

    if not web_concurrency_str:
        cores = multiprocessing.cpu_count()
        workers_per_core = float(workers_per_core_str)
        default_web_concurrency = workers_per_core * cores
        web_concurrency = max(int(default_web_concurrency), 2)
        if use_max_workers:
            web_concurrency = min(web_concurrency, use_max_workers)
        web_concurrency_str = str(web_concurrency)
    os.environ["WEB_CONCURRENCY"] = web_concurrency_str
    logging.warning(
        f'Set the number of model server workers to {os.environ["WEB_CONCURRENCY"]}.'
    )


if __name__ == "__main__":
    set_number_of_workers_from_env()
    uvicorn.run(
        "google.cloud.aiplatform.prediction.model_server:CprModelServer",
        host="0.0.0.0",
        port=int(os.environ.get("AIP_HTTP_PORT")),
        factory=True,
    )



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from typing import MutableMapping, MutableSequence

import proto  # type: ignore

from google.cloud.aiplatform_v1beta1.types import encryption_spec as gca_encryption_spec
from google.protobuf import timestamp_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.cloud.aiplatform.v1beta1",
    manifest={
        "Tensorboard",
    },
)


class Tensorboard(proto.Message):
    r"""Tensorboard is a physical database that stores users'
    training metrics. A default Tensorboard is provided in each
    region of a Google Cloud project. If needed users can also
    create extra Tensorboards in their projects.

    Attributes:
        name (str):
            Output only. Name of the Tensorboard. Format:
            ``projects/{project}/locations/{location}/tensorboards/{tensorboard}``
        display_name (str):
            Required. User provided name of this
            Tensorboard.
        description (str):
            Description of this Tensorboard.
        encryption_spec (google.cloud.aiplatform_v1beta1.types.EncryptionSpec):
            Customer-managed encryption key spec for a
            Tensorboard. If set, this Tensorboard and all
            sub-resources of this Tensorboard will be
            secured by this key.
        blob_storage_path_prefix (str):
            Output only. Consumer project Cloud Storage
            path prefix used to store blob data, which can
            either be a bucket or directory. Does not end
            with a '/'.
        run_count (int):
            Output only. The number of Runs stored in
            this Tensorboard.
        create_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Timestamp when this Tensorboard
            was created.
        update_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Timestamp when this Tensorboard
            was last updated.
        labels (MutableMapping[str, str]):
            The labels with user-defined metadata to
            organize your Tensorboards.
            Label keys and values can be no longer than 64
            characters (Unicode codepoints), can only
            contain lowercase letters, numeric characters,
            underscores and dashes. International characters
            are allowed. No more than 64 user labels can be
            associated with one Tensorboard (System labels
            are excluded).

            See https://goo.gl/xmQnxf for more information
            and examples of labels. System reserved label
            keys are prefixed with
            "aiplatform.googleapis.com/" and are immutable.
        etag (str):
            Used to perform a consistent
            read-modify-write updates. If not set, a blind
            "overwrite" update happens.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    display_name: str = proto.Field(
        proto.STRING,
        number=2,
    )
    description: str = proto.Field(
        proto.STRING,
        number=3,
    )
    encryption_spec: gca_encryption_spec.EncryptionSpec = proto.Field(
        proto.MESSAGE,
        number=11,
        message=gca_encryption_spec.EncryptionSpec,
    )
    blob_storage_path_prefix: str = proto.Field(
        proto.STRING,
        number=10,
    )
    run_count: int = proto.Field(
        proto.INT32,
        number=5,
    )
    create_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=6,
        message=timestamp_pb2.Timestamp,
    )
    update_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=7,
        message=timestamp_pb2.Timestamp,
    )
    labels: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=8,
    )
    etag: str = proto.Field(
        proto.STRING,
        number=9,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from typing import MutableMapping, MutableSequence

import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.cloud.aiplatform.v1.schema.predict.instance",
    manifest={
        "TextClassificationPredictionInstance",
    },
)


class TextClassificationPredictionInstance(proto.Message):
    r"""Prediction input format for Text Classification.

    Attributes:
        content (str):
            The text snippet to make the predictions on.
        mime_type (str):
            The MIME type of the text snippet. The
            supported MIME types are listed below.
            - text/plain
    """

    content: str = proto.Field(
        proto.STRING,
        number=1,
    )
    mime_type: str = proto.Field(
        proto.STRING,
        number=2,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



"""Convenience classes for configuring Vizier Early-Stopping Configs."""
import copy
from typing import Union

import attr

from google.cloud.aiplatform.compat.types import study as study_pb2

AutomatedStoppingConfigProto = Union[
    study_pb2.StudySpec.DecayCurveAutomatedStoppingSpec,
    study_pb2.StudySpec.MedianAutomatedStoppingSpec,
]


@attr.s(frozen=True, init=True, slots=True, kw_only=True)
class AutomatedStoppingConfig:
    """A wrapper for study_pb2.automated_stopping_spec."""

    _proto: AutomatedStoppingConfigProto = attr.ib(init=True, kw_only=True)

    @classmethod
    def decay_curve_stopping_config(cls, use_steps: bool) -> "AutomatedStoppingConfig":
        """Create a DecayCurve automated stopping config.

        Vizier will early stop the Trial if it predicts the Trial objective value
        will not be better than previous Trials.

        Args:
          use_steps: Bool. If set, use Measurement.step_count as the measure of
            training progress.  Otherwise, use Measurement.elapsed_duration.

        Returns:
          AutomatedStoppingConfig object.

        Raises:
          ValueError: If more than one metric is configured.
          Note that Vizier Early Stopping currently only supports single-objective
          studies.
        """
        config = study_pb2.StudySpec.DecayCurveAutomatedStoppingSpec(
            use_elapsed_duration=not use_steps
        )
        return cls(proto=config)

    @classmethod
    def median_automated_stopping_config(
        cls, use_steps: bool
    ) -> "AutomatedStoppingConfig":
        """Create a Median automated stopping config.

        Vizier will early stop the Trial if it predicts the Trial objective value
        will not be better than previous Trials.

        Args:
          use_steps: Bool. If set, use Measurement.step_count as the measure of
            training progress.  Otherwise, use Measurement.elapsed_duration.

        Returns:
          AutomatedStoppingConfig object.

        Raises:
          ValueError: If more than one metric is configured.
          Note that Vizier Early Stopping currently only supports single-objective
          studies.
        """
        config = study_pb2.StudySpec.MedianAutomatedStoppingSpec(
            use_elapsed_duration=not use_steps
        )
        return cls(proto=config)

    @classmethod
    def from_proto(
        cls, proto: AutomatedStoppingConfigProto
    ) -> "AutomatedStoppingConfig":
        return cls(proto=proto)

    def to_proto(self) -> AutomatedStoppingConfigProto:
        """Returns this object as a proto."""
        return copy.deepcopy(self._proto)



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from typing import MutableMapping, MutableSequence

import proto  # type: ignore

from google.cloud.aiplatform_v1beta1.types import artifact
from google.cloud.aiplatform_v1beta1.types import context
from google.cloud.aiplatform_v1beta1.types import encryption_spec as gca_encryption_spec
from google.cloud.aiplatform_v1beta1.types import execution as gca_execution
from google.cloud.aiplatform_v1beta1.types import pipeline_failure_policy
from google.cloud.aiplatform_v1beta1.types import pipeline_state
from google.cloud.aiplatform_v1beta1.types import value as gca_value
from google.protobuf import struct_pb2  # type: ignore
from google.protobuf import timestamp_pb2  # type: ignore
from google.rpc import status_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.cloud.aiplatform.v1beta1",
    manifest={
        "PipelineJob",
        "PipelineTemplateMetadata",
        "PipelineJobDetail",
        "PipelineTaskDetail",
        "PipelineTaskExecutorDetail",
    },
)


class PipelineJob(proto.Message):
    r"""An instance of a machine learning PipelineJob.

    Attributes:
        name (str):
            Output only. The resource name of the
            PipelineJob.
        display_name (str):
            The display name of the Pipeline.
            The name can be up to 128 characters long and
            can consist of any UTF-8 characters.
        create_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Pipeline creation time.
        start_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Pipeline start time.
        end_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Pipeline end time.
        update_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Timestamp when this PipelineJob
            was most recently updated.
        pipeline_spec (google.protobuf.struct_pb2.Struct):
            The spec of the pipeline.
        state (google.cloud.aiplatform_v1beta1.types.PipelineState):
            Output only. The detailed state of the job.
        job_detail (google.cloud.aiplatform_v1beta1.types.PipelineJobDetail):
            Output only. The details of pipeline run. Not
            available in the list view.
        error (google.rpc.status_pb2.Status):
            Output only. The error that occurred during
            pipeline execution. Only populated when the
            pipeline's state is FAILED or CANCELLED.
        labels (MutableMapping[str, str]):
            The labels with user-defined metadata to
            organize PipelineJob.
            Label keys and values can be no longer than 64
            characters (Unicode codepoints), can only
            contain lowercase letters, numeric characters,
            underscores and dashes. International characters
            are allowed.
            See https://goo.gl/xmQnxf for more information
            and examples of labels.
        runtime_config (google.cloud.aiplatform_v1beta1.types.PipelineJob.RuntimeConfig):
            Runtime config of the pipeline.
        encryption_spec (google.cloud.aiplatform_v1beta1.types.EncryptionSpec):
            Customer-managed encryption key spec for a
            pipelineJob. If set, this PipelineJob and all of
            its sub-resources will be secured by this key.
        service_account (str):
            The service account that the pipeline workload runs as. If
            not specified, the Compute Engine default service account in
            the project will be used. See
            https://cloud.google.com/compute/docs/access/service-accounts#default_service_account

            Users starting the pipeline must have the
            ``iam.serviceAccounts.actAs`` permission on this service
            account.
        network (str):
            The full name of the Compute Engine
            `network </compute/docs/networks-and-firewalls#networks>`__
            to which the Pipeline Job's workload should be peered. For
            example, ``projects/12345/global/networks/myVPC``.
            `Format </compute/docs/reference/rest/v1/networks/insert>`__
            is of the form
            ``projects/{project}/global/networks/{network}``. Where
            {project} is a project number, as in ``12345``, and
            {network} is a network name.

            Private services access must already be configured for the
            network. Pipeline job will apply the network configuration
            to the Google Cloud resources being launched, if applied,
            such as Vertex AI Training or Dataflow job. If left
            unspecified, the workload is not peered with any network.
        template_uri (str):
            A template uri from where the
            [PipelineJob.pipeline_spec][google.cloud.aiplatform.v1beta1.PipelineJob.pipeline_spec],
            if empty, will be downloaded.
        template_metadata (google.cloud.aiplatform_v1beta1.types.PipelineTemplateMetadata):
            Output only. Pipeline template metadata. Will fill up fields
            if
            [PipelineJob.template_uri][google.cloud.aiplatform.v1beta1.PipelineJob.template_uri]
            is from supported template registry.
    """

    class RuntimeConfig(proto.Message):
        r"""The runtime config of a PipelineJob.

        Attributes:
            parameters (MutableMapping[str, google.cloud.aiplatform_v1beta1.types.Value]):
                Deprecated. Use
                [RuntimeConfig.parameter_values][google.cloud.aiplatform.v1beta1.PipelineJob.RuntimeConfig.parameter_values]
                instead. The runtime parameters of the PipelineJob. The
                parameters will be passed into
                [PipelineJob.pipeline_spec][google.cloud.aiplatform.v1beta1.PipelineJob.pipeline_spec]
                to replace the placeholders at runtime. This field is used
                by pipelines built using
                ``PipelineJob.pipeline_spec.schema_version`` 2.0.0 or lower,
                such as pipelines built using Kubeflow Pipelines SDK 1.8 or
                lower.
            gcs_output_directory (str):
                Required. A path in a Cloud Storage bucket, which will be
                treated as the root output directory of the pipeline. It is
                used by the system to generate the paths of output
                artifacts. The artifact paths are generated with a sub-path
                pattern ``{job_id}/{task_id}/{output_key}`` under the
                specified output directory. The service account specified in
                this pipeline must have the ``storage.objects.get`` and
                ``storage.objects.create`` permissions for this bucket.
            parameter_values (MutableMapping[str, google.protobuf.struct_pb2.Value]):
                The runtime parameters of the PipelineJob. The parameters
                will be passed into
                [PipelineJob.pipeline_spec][google.cloud.aiplatform.v1beta1.PipelineJob.pipeline_spec]
                to replace the placeholders at runtime. This field is used
                by pipelines built using
                ``PipelineJob.pipeline_spec.schema_version`` 2.1.0, such as
                pipelines built using Kubeflow Pipelines SDK 1.9 or higher
                and the v2 DSL.
            failure_policy (google.cloud.aiplatform_v1beta1.types.PipelineFailurePolicy):
                Represents the failure policy of a pipeline. Currently, the
                default of a pipeline is that the pipeline will continue to
                run until no more tasks can be executed, also known as
                PIPELINE_FAILURE_POLICY_FAIL_SLOW. However, if a pipeline is
                set to PIPELINE_FAILURE_POLICY_FAIL_FAST, it will stop
                scheduling any new tasks when a task has failed. Any
                scheduled tasks will continue to completion.
            input_artifacts (MutableMapping[str, google.cloud.aiplatform_v1beta1.types.PipelineJob.RuntimeConfig.InputArtifact]):
                The runtime artifacts of the PipelineJob. The
                key will be the input artifact name and the
                value would be one of the InputArtifact.
        """

        class InputArtifact(proto.Message):
            r"""The type of an input artifact.

            .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

            Attributes:
                artifact_id (str):
                    Artifact resource id from MLMD. Which is the last portion of
                    an artifact resource name:
                    ``projects/{project}/locations/{location}/metadataStores/default/artifacts/{artifact_id}``.
                    The artifact must stay within the same project, location and
                    default metadatastore as the pipeline.

                    This field is a member of `oneof`_ ``kind``.
            """

            artifact_id: str = proto.Field(
                proto.STRING,
                number=1,
                oneof="kind",
            )

        parameters: MutableMapping[str, gca_value.Value] = proto.MapField(
            proto.STRING,
            proto.MESSAGE,
            number=1,
            message=gca_value.Value,
        )
        gcs_output_directory: str = proto.Field(
            proto.STRING,
            number=2,
        )
        parameter_values: MutableMapping[str, struct_pb2.Value] = proto.MapField(
            proto.STRING,
            proto.MESSAGE,
            number=3,
            message=struct_pb2.Value,
        )
        failure_policy: pipeline_failure_policy.PipelineFailurePolicy = proto.Field(
            proto.ENUM,
            number=4,
            enum=pipeline_failure_policy.PipelineFailurePolicy,
        )
        input_artifacts: MutableMapping[
            str, "PipelineJob.RuntimeConfig.InputArtifact"
        ] = proto.MapField(
            proto.STRING,
            proto.MESSAGE,
            number=5,
            message="PipelineJob.RuntimeConfig.InputArtifact",
        )

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    display_name: str = proto.Field(
        proto.STRING,
        number=2,
    )
    create_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )
    start_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=4,
        message=timestamp_pb2.Timestamp,
    )
    end_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=5,
        message=timestamp_pb2.Timestamp,
    )
    update_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=6,
        message=timestamp_pb2.Timestamp,
    )
    pipeline_spec: struct_pb2.Struct = proto.Field(
        proto.MESSAGE,
        number=7,
        message=struct_pb2.Struct,
    )
    state: pipeline_state.PipelineState = proto.Field(
        proto.ENUM,
        number=8,
        enum=pipeline_state.PipelineState,
    )
    job_detail: "PipelineJobDetail" = proto.Field(
        proto.MESSAGE,
        number=9,
        message="PipelineJobDetail",
    )
    error: status_pb2.Status = proto.Field(
        proto.MESSAGE,
        number=10,
        message=status_pb2.Status,
    )
    labels: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=11,
    )
    runtime_config: RuntimeConfig = proto.Field(
        proto.MESSAGE,
        number=12,
        message=RuntimeConfig,
    )
    encryption_spec: gca_encryption_spec.EncryptionSpec = proto.Field(
        proto.MESSAGE,
        number=16,
        message=gca_encryption_spec.EncryptionSpec,
    )
    service_account: str = proto.Field(
        proto.STRING,
        number=17,
    )
    network: str = proto.Field(
        proto.STRING,
        number=18,
    )
    template_uri: str = proto.Field(
        proto.STRING,
        number=19,
    )
    template_metadata: "PipelineTemplateMetadata" = proto.Field(
        proto.MESSAGE,
        number=20,
        message="PipelineTemplateMetadata",
    )


class PipelineTemplateMetadata(proto.Message):
    r"""Pipeline template metadata if
    [PipelineJob.template_uri][google.cloud.aiplatform.v1beta1.PipelineJob.template_uri]
    is from supported template registry. Currently, the only supported
    registry is Artifact Registry.

    Attributes:
        version (str):
            The version_name in artifact registry.

            Will always be presented in output if the
            [PipelineJob.template_uri][google.cloud.aiplatform.v1beta1.PipelineJob.template_uri]
            is from supported template registry.

            Format is "sha256:abcdef123456...".
    """

    version: str = proto.Field(
        proto.STRING,
        number=3,
    )


class PipelineJobDetail(proto.Message):
    r"""The runtime detail of PipelineJob.

    Attributes:
        pipeline_context (google.cloud.aiplatform_v1beta1.types.Context):
            Output only. The context of the pipeline.
        pipeline_run_context (google.cloud.aiplatform_v1beta1.types.Context):
            Output only. The context of the current
            pipeline run.
        task_details (MutableSequence[google.cloud.aiplatform_v1beta1.types.PipelineTaskDetail]):
            Output only. The runtime details of the tasks
            under the pipeline.
    """

    pipeline_context: context.Context = proto.Field(
        proto.MESSAGE,
        number=1,
        message=context.Context,
    )
    pipeline_run_context: context.Context = proto.Field(
        proto.MESSAGE,
        number=2,
        message=context.Context,
    )
    task_details: MutableSequence["PipelineTaskDetail"] = proto.RepeatedField(
        proto.MESSAGE,
        number=3,
        message="PipelineTaskDetail",
    )


class PipelineTaskDetail(proto.Message):
    r"""The runtime detail of a task execution.

    Attributes:
        task_id (int):
            Output only. The system generated ID of the
            task.
        parent_task_id (int):
            Output only. The id of the parent task if the
            task is within a component scope. Empty if the
            task is at the root level.
        task_name (str):
            Output only. The user specified name of the task that is
            defined in [PipelineJob.spec][].
        create_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Task create time.
        start_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Task start time.
        end_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Task end time.
        executor_detail (google.cloud.aiplatform_v1beta1.types.PipelineTaskExecutorDetail):
            Output only. The detailed execution info.
        state (google.cloud.aiplatform_v1beta1.types.PipelineTaskDetail.State):
            Output only. State of the task.
        execution (google.cloud.aiplatform_v1beta1.types.Execution):
            Output only. The execution metadata of the
            task.
        error (google.rpc.status_pb2.Status):
            Output only. The error that occurred during
            task execution. Only populated when the task's
            state is FAILED or CANCELLED.
        pipeline_task_status (MutableSequence[google.cloud.aiplatform_v1beta1.types.PipelineTaskDetail.PipelineTaskStatus]):
            Output only. A list of task status. This
            field keeps a record of task status evolving
            over time.
        inputs (MutableMapping[str, google.cloud.aiplatform_v1beta1.types.PipelineTaskDetail.ArtifactList]):
            Output only. The runtime input artifacts of
            the task.
        outputs (MutableMapping[str, google.cloud.aiplatform_v1beta1.types.PipelineTaskDetail.ArtifactList]):
            Output only. The runtime output artifacts of
            the task.
    """

    class State(proto.Enum):
        r"""Specifies state of TaskExecution

        Values:
            STATE_UNSPECIFIED (0):
                Unspecified.
            PENDING (1):
                Specifies pending state for the task.
            RUNNING (2):
                Specifies task is being executed.
            SUCCEEDED (3):
                Specifies task completed successfully.
            CANCEL_PENDING (4):
                Specifies Task cancel is in pending state.
            CANCELLING (5):
                Specifies task is being cancelled.
            CANCELLED (6):
                Specifies task was cancelled.
            FAILED (7):
                Specifies task failed.
            SKIPPED (8):
                Specifies task was skipped due to cache hit.
            NOT_TRIGGERED (9):
                Specifies that the task was not triggered because the task's
                trigger policy is not satisfied. The trigger policy is
                specified in the ``condition`` field of
                [PipelineJob.pipeline_spec][google.cloud.aiplatform.v1beta1.PipelineJob.pipeline_spec].
        """
        STATE_UNSPECIFIED = 0
        PENDING = 1
        RUNNING = 2
        SUCCEEDED = 3
        CANCEL_PENDING = 4
        CANCELLING = 5
        CANCELLED = 6
        FAILED = 7
        SKIPPED = 8
        NOT_TRIGGERED = 9

    class PipelineTaskStatus(proto.Message):
        r"""A single record of the task status.

        Attributes:
            update_time (google.protobuf.timestamp_pb2.Timestamp):
                Output only. Update time of this status.
            state (google.cloud.aiplatform_v1beta1.types.PipelineTaskDetail.State):
                Output only. The state of the task.
            error (google.rpc.status_pb2.Status):
                Output only. The error that occurred during
                the state. May be set when the state is any of
                the non-final state (PENDING/RUNNING/CANCELLING)
                or FAILED state. If the state is FAILED, the
                error here is final and not going to be retried.
                If the state is a non-final state, the error
                indicates a system-error being retried.
        """

        update_time: timestamp_pb2.Timestamp = proto.Field(
            proto.MESSAGE,
            number=1,
            message=timestamp_pb2.Timestamp,
        )
        state: "PipelineTaskDetail.State" = proto.Field(
            proto.ENUM,
            number=2,
            enum="PipelineTaskDetail.State",
        )
        error: status_pb2.Status = proto.Field(
            proto.MESSAGE,
            number=3,
            message=status_pb2.Status,
        )

    class ArtifactList(proto.Message):
        r"""A list of artifact metadata.

        Attributes:
            artifacts (MutableSequence[google.cloud.aiplatform_v1beta1.types.Artifact]):
                Output only. A list of artifact metadata.
        """

        artifacts: MutableSequence[artifact.Artifact] = proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message=artifact.Artifact,
        )

    task_id: int = proto.Field(
        proto.INT64,
        number=1,
    )
    parent_task_id: int = proto.Field(
        proto.INT64,
        number=12,
    )
    task_name: str = proto.Field(
        proto.STRING,
        number=2,
    )
    create_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )
    start_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=4,
        message=timestamp_pb2.Timestamp,
    )
    end_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=5,
        message=timestamp_pb2.Timestamp,
    )
    executor_detail: "PipelineTaskExecutorDetail" = proto.Field(
        proto.MESSAGE,
        number=6,
        message="PipelineTaskExecutorDetail",
    )
    state: State = proto.Field(
        proto.ENUM,
        number=7,
        enum=State,
    )
    execution: gca_execution.Execution = proto.Field(
        proto.MESSAGE,
        number=8,
        message=gca_execution.Execution,
    )
    error: status_pb2.Status = proto.Field(
        proto.MESSAGE,
        number=9,
        message=status_pb2.Status,
    )
    pipeline_task_status: MutableSequence[PipelineTaskStatus] = proto.RepeatedField(
        proto.MESSAGE,
        number=13,
        message=PipelineTaskStatus,
    )
    inputs: MutableMapping[str, ArtifactList] = proto.MapField(
        proto.STRING,
        proto.MESSAGE,
        number=10,
        message=ArtifactList,
    )
    outputs: MutableMapping[str, ArtifactList] = proto.MapField(
        proto.STRING,
        proto.MESSAGE,
        number=11,
        message=ArtifactList,
    )


class PipelineTaskExecutorDetail(proto.Message):
    r"""The runtime detail of a pipeline executor.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        container_detail (google.cloud.aiplatform_v1beta1.types.PipelineTaskExecutorDetail.ContainerDetail):
            Output only. The detailed info for a
            container executor.

            This field is a member of `oneof`_ ``details``.
        custom_job_detail (google.cloud.aiplatform_v1beta1.types.PipelineTaskExecutorDetail.CustomJobDetail):
            Output only. The detailed info for a custom
            job executor.

            This field is a member of `oneof`_ ``details``.
    """

    class ContainerDetail(proto.Message):
        r"""The detail of a container execution. It contains the job
        names of the lifecycle of a container execution.

        Attributes:
            main_job (str):
                Output only. The name of the
                [CustomJob][google.cloud.aiplatform.v1beta1.CustomJob] for
                the main container execution.
            pre_caching_check_job (str):
                Output only. The name of the
                [CustomJob][google.cloud.aiplatform.v1beta1.CustomJob] for
                the pre-caching-check container execution. This job will be
                available if the
                [PipelineJob.pipeline_spec][google.cloud.aiplatform.v1beta1.PipelineJob.pipeline_spec]
                specifies the ``pre_caching_check`` hook in the lifecycle
                events.
            failed_main_jobs (MutableSequence[str]):
                Output only. The names of the previously failed
                [CustomJob][google.cloud.aiplatform.v1beta1.CustomJob] for
                the main container executions. The list includes the all
                attempts in chronological order.
            failed_pre_caching_check_jobs (MutableSequence[str]):
                Output only. The names of the previously failed
                [CustomJob][google.cloud.aiplatform.v1beta1.CustomJob] for
                the pre-caching-check container executions. This job will be
                available if the
                [PipelineJob.pipeline_spec][google.cloud.aiplatform.v1beta1.PipelineJob.pipeline_spec]
                specifies the ``pre_caching_check`` hook in the lifecycle
                events. The list includes the all attempts in chronological
                order.
        """

        main_job: str = proto.Field(
            proto.STRING,
            number=1,
        )
        pre_caching_check_job: str = proto.Field(
            proto.STRING,
            number=2,
        )
        failed_main_jobs: MutableSequence[str] = proto.RepeatedField(
            proto.STRING,
            number=3,
        )
        failed_pre_caching_check_jobs: MutableSequence[str] = proto.RepeatedField(
            proto.STRING,
            number=4,
        )

    class CustomJobDetail(proto.Message):
        r"""The detailed info for a custom job executor.

        Attributes:
            job (str):
                Output only. The name of the
                [CustomJob][google.cloud.aiplatform.v1beta1.CustomJob].
            failed_jobs (MutableSequence[str]):
                Output only. The names of the previously failed
                [CustomJob][google.cloud.aiplatform.v1beta1.CustomJob]. The
                list includes the all attempts in chronological order.
        """

        job: str = proto.Field(
            proto.STRING,
            number=1,
        )
        failed_jobs: MutableSequence[str] = proto.RepeatedField(
            proto.STRING,
            number=3,
        )

    container_detail: ContainerDetail = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="details",
        message=ContainerDetail,
    )
    custom_job_detail: CustomJobDetail = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="details",
        message=CustomJobDetail,
    )


__all__ = tuple(sorted(__protobuf__.manifest))
